/*++

  COPYRIGHT        (C) NEC        Corporation 1997 - 1999

  All rights reserved by NEC Corporation. This program must be
  used solely for the purpose for which it was furnished by NEC
  Corporation. No part of this program may be reproduced or
  disclosed to others, in any form, without the prior written
  permission of NEC Corporation.
  Use of copyright notice does not evidence publication of this
  program.

  Author	:  		T.Muto(NEC Software Ltd.,)
  Module Name:		kmutil.h

  Abstract:
	KAMUI = Kamui is abstruction model that is ultimate interface for PowerVR.
	'kmutil.h' utilities functions for KAMUI.

  Environment:
	Microsoft Compiler.

  Notes:

  Revision History:

--*/

#if !defined(__kamui_utility_h)
#define __kamui_utility_h

#ifdef __cplusplus
extern "C" {
#endif 

#include "kamui2.h"

/*
 * KMUPASSSTATUS is kmuXxxPassTable Status.
 */
typedef KMUINT32	KMUPASSSTATUS;

/*
 *	for kmutil target
 */
#define	KMU_TARGET_ARC1			(0x00000001)
#define	KMU_TARGET_CLX1			(0x00000002)
#define	KMU_TARGET_CLX2			(0x00000003)

/*
 *	for kmuXxxPassTable
 */
#define KMU_PASS_OK							 0L
#define KMU_PASS_ERROR_VERTEXCONTEXT		 1L
#define KMU_PASS_ERROR_VERTEXCONTEXT_PASS	 2L
#define KMU_PASS_ERROR_PARAMTYPE			 3L
#define KMU_PASS_ERROR_LISTTYPE				 4L
#define KMU_PASS_ERROR_MIPMAP_D_ADJUST		 5L
#define KMU_PASS_ERROR_FOGMODE				 6L
#define KMU_PASS_ERROR_FILTERMODE			 7L
#define KMU_PASS_ERROR_TEXTURESHADINGMODE	 8L
#define KMU_PASS_ERROR_COLORTYPE			 9L
#define KMU_PASS_ERROR_SHADINGMODE			10L
#define KMU_PASS_ERROR_USERCLIPMODE			11L
#define KMU_PASS_ERROR_TRILINEAR_SETTING	12L
#define KMU_PASS_ERROR_SPRITE_SETTING		13L
#define KMU_PASS_ERROR_BLENDINGMODE_SETTING	14L
#define KMU_PASS_ERROR_MODIFIER_SETTING		15L

#define KMU_PASS_ERROR_NOT_IMPLEMENTED	   120L

/*
 * for kmuCalculateKamuiWorkareaSize
 */
typedef struct tagKMWORKAREASIZE
{
	/* Inputs by user */
	KMDWORD		dwNumberOfFameBuffes;		/* Number of Frame buffers */
	KMDWORD		dwNumberOfNativeBuffers;	/* Number of Native buffer */
	KMDWORD		dwNumberOfTextures;			/* Number of Textures (No small VQ) */
	KMDWORD		dwNumberOf8x8SmallVQ;		/* Number of 8x8 small VQ texture */
	KMDWORD		dwNumberOf8x8SmallVQmm;		/* Number of 8x8 small VQ mipmap texture */
	KMDWORD		dwNumberOf16x16SmallVQ;		/* Number of 16x16 small VQ texture */
	KMDWORD		dwNumberOf16x16SmallVQmm;	/* Number of 16x16 small VQ mipmap texture */
	KMDWORD		dwNumberOf32x32SmallVQ;		/* Number of 32x32 small VQ texture */
	KMDWORD		dwNumberOf32x32SmallVQmm;	/* Number of 32x32 small VQ mipmap texture */
	KMDWORD		dwNumberOf64x64SmallVQ;		/* Number of 64x64 small VQ texture */
	KMDWORD		dwNumberOf64x64SmallVQmm;	/* Number of 64x64 small VQ mipmap texture */

	/* Outputs for SystemConfigStruct */
	KMDWORD		dwTextureStructNum;			/* Number of texture structure */
	KMDWORD		dwSmallVQStructNum;			/* Number of small VQ texture structure */
	KMDWORD		dwKamuiWorkareaSize;		/* Size of Kamui work area */
} KMWORKAREASIZE, *PKMWORKAREASIZE;


/*++
kmuCreateTwiddledTexture

proto type:

	KMSTATUS
	kmuCreateTwiddledTexture(
			PKMDWORD		pOutputTexture,
			PKMDWORD		pInputTexture,
			KMBOOLEAN		bAutoMipMap,
			KMBOOLEAN		bDither,
			KMINT32			USize,
			KMTEXTURETYPE	nTextureType
		);

Description:
	CKM_TEXTURE_BMP`(ABGR8888)̃eNX`A
	Twiddled / Twiddled Mipmap `̃eNX`ɕϊ܂B
	bAutoMipMapTRUEw肵ꍇ́A~bv}bv̎s܂B
	bDitherTRUEw肵ꍇ́AfBUs܂B
	ӁF~bv}bvAfBU̎wsƂ
			̓eNX`f[^̓e͔j󂳂܂B

Arguments:
	OUT	pOutputTexture	ϊ̃eNX`f[^ރC̃AhXłB
	IN	pInputTexture	KM_TEXTURE_BMP`̓̓eNX`|C^łB
	IN	bAutoMipMap		IMIPMAP𐶐邩ǂw肵܂B
						TRUEw肳ꂽꍇɂ͎IMIPMAP𐶐܂B
						(o͂KM_TEXTURE_TWIDDLED_MM`ɂȂ܂)
						FALSEw肳ꂽꍇɂMIPMAP͍s܂B
						(o͂KM_TEXTURE_TWIDDLED`ɂȂ܂)
	IN	bDither			fBU邩ǂw肵܂B
						TRUEw肳ꂽꍇ̓fBU܂B
	IN	USize			eNX`̈ӂ̃eNZw肵܂B
						L̒̂ꂩw肵܂B
						KM_MAPSIZE_8
						KM_MAPSIZE_16
						KM_MAPSIZE_32
						KM_MAPSIZE_64
						KM_MAPSIZE_128
						KM_MAPSIZE_256
						KM_MAPSIZE_512
						KM_MAPSIZE_1024
	IN	nTextureType	ϊ̃eNX`̃sNZtH[}bgw肵܂B
						L̒̂ꂩw肵܂B
						KM_PIXELFORMAT_ARGB1555
						KM_PIXELFORMAT_RGB565
						KM_PIXELFORMAT_ARGB4444

Result:
	KMSTATUS_SUCCESS					ǂݍݐ
	KMSTATUS_INVALID_TEXTURE_TYPE		ȃeNX`^CvATCYw肵

--*/

KMSTATUS KMAPI
kmuCreateTwiddledTexture(
	OUT	PKMDWORD		pOutputTexture,
	IN	PKMDWORD		pInputTexture,
	IN	KMBOOLEAN		bAutoMipMap,
	IN	KMBOOLEAN		bDither,
	IN	KMINT32			USize,
	IN	KMTEXTURETYPE	nTextureType
	);


/*++
kmuCreateTwiddledTextureEx

proto type:

	KMSTATUS
	kmuCreateTwiddledTextureEx(
		OUT	PKMDWORD		pOutputTexture,
		IN	PKMDWORD		pInputTexture,
		IN	KMBOOLEAN		bAutoMipMap,
		IN	KMBOOLEAN		bDither,
		IN	KMINT32			USize,
		IN	KMINT32			VSize,
		IN	KMTEXTURETYPE	nTextureType,
		IN	PKMDWORD		pWorkarea
		);

Description:
	CKM_TEXTURE_BMP`(ABGR8888)̃eNX`A
	Twiddled / Twiddled Mipmap `̃eNX`ɕϊ܂B
	bAutoMipMapTRUEw肵ꍇ́A~bv}bv̎s܂B
	bDitherTRUEw肵ꍇ́AfBUs܂B
	USizeVSizeɈقȂlw肵ꍇɂ́ATwiddledRectangle`
	̃eNX`𐶐܂B
	ӁF~bv}bvAfBU̎wsƂ
			̓eNX`f[^̓e͔j󂳂܂B

Arguments:
	OUT	pOutputTexture	ϊ̃eNX`f[^ރC̃AhXłB
	IN	pInputTexture	KM_TEXTURE_BMP`̓̓eNX`|C^łB
	IN	bAutoMipMap		IMIPMAP𐶐邩ǂw肵܂B
						TRUEw肳ꂽꍇɂ͎IMIPMAP𐶐܂B
						(o͂KM_TEXTURE_TWIDDLED_MM`ɂȂ܂)
						FALSEw肳ꂽꍇɂMIPMAP͍s܂B
						(o͂KM_TEXTURE_TWIDDLED`ɂȂ܂)
	IN	bDither			fBU邩ǂw肵܂B
						TRUEw肳ꂽꍇ̓fBU܂B
	IN	USize, VSize	eNX`̈ӂ̃eNZw肵܂B
						L̒̂ꂩw肵܂B
						KM_MAPSIZE_8
						KM_MAPSIZE_16
						KM_MAPSIZE_32
						KM_MAPSIZE_64
						KM_MAPSIZE_128
						KM_MAPSIZE_256
						KM_MAPSIZE_512
						KM_MAPSIZE_1024
	IN	nTextureType	ϊ̃eNX`̃sNZtH[}bgw肵܂B
						L̒̂ꂩw肵܂B
						KM_PIXELFORMAT_ARGB1555
						KM_PIXELFORMAT_RGB565
						KM_PIXELFORMAT_ARGB4444
	IN	pWorkarea		̓f[^ƓTCỸ[NGAA
						̐擪AhXw肵܂B
						̃eNX`Twiddled-Rectangle`ɕϊꍇɕKvłB 
						̑̏ꍇɂ̓[NGA͕Kv܂B
						̏ꍇɂNULLw肵ĂB 

Result:
	KMSTATUS_SUCCESS					ǂݍݐ
	KMSTATUS_INVALID_TEXTURE_TYPE		ȃeNX`^CvATCYw肵

--*/

KMSTATUS KMAPI
kmuCreateTwiddledTextureEx(
	OUT	PKMDWORD		pOutputTexture,
	IN	PKMDWORD		pInputTexture,
	IN	KMBOOLEAN		bAutoMipMap,
	IN	KMBOOLEAN		bDither,
	IN	KMINT32			USize,
	IN	KMINT32			VSize,
	IN	KMTEXTURETYPE	nTextureType,
	IN	PKMDWORD		pWorkarea
	);


/*++
kmuConvertFBtoBMP

proto type:

	KMSTATUS
	kmuConvertFBtoBMP(
			PKMDWORD		pOutputData,
			PKMDWORD		pInputData,
			KMINT32			nWidth,
			KMINT32			nHeight,
			KMBPPMODE		nBpp
		);

Description:
	kmGetTextureŃCɓǂݏot[obt@̓eA
	Windows tJ[BMP`̃sNZf[^ƂāAɏo܂B
	ȂȀo͂ɂWindows BMP̃wb_54oCg͐܂B

Arguments:
	OUT	pOutputData		ϊ̃sNZf[^ރC̃AhXłB
	IN	pInputData		t[obt@̓e|C^łB
						kmGetTextureŁAt[obt@T[tFXւ̃fBXNv^
						w肵ēǂݏot[obt@̃sNZf[^wĂȂ
						܂BKAMUIeNX`f[^͎ws\łB
	IN	nWidth,nHeight	ǂݏot[obt@̃TCYw肵܂B
	IN	nBpp			ǂݏot[obt@̃sNZtH[}bgw肵܂B
						L̒̂ꂩw肵܂B
							KM_DSPBPP_RGB565
							KM_DSPBPP_RGB555
							KM_DSPBPP_ARGB4444
							KM_DSPBPP_ARGB1555

Result:
	KMSTATUS_SUCCESS					ǂݍݐ

--*/

KMSTATUS KMAPI
kmuConvertFBtoBMP(
		PKMDWORD		pOutputData,
		PKMDWORD		pInputData,
		KMINT32			nWidth,
		KMINT32			nHeight,
		KMBPPMODE		nBpp
		);


/*++
kmuCalculateKamuiWorkareaSize

proto type:

	KMSTATUS
	kmuCalculateKamuiWorkareaSize(
			IN OUT PKMWORKAREASIZE	pWorkareaSize
		);

Description:
	KMWORKAREASIZE\̂Ɏw肳ꂽep[^A
	SystemConfigStruct  dwTextureStructNum, dwSmallVQStructNum o
	̓eAy pAddress oɎw肷郏[NGÄ̃TCY
	߂܂B

Arguments:
	IN OUT PKMWORKAREASIZE	pWorkareaSize
	̓p[^yьʂۑ\̂łB

	dwNumberOfFameBuffes
	t[obt@̐w肵܂BႦ΁A_uobt@̏ꍇ͂Qw肵܂B
	dwNumberOfNativeBuffers
	lCeBuR}hobt@̐w肵܂BႦ΁A_uobt@̏ꍇ͂Qw肵܂B
	KMDWORD		dwNumberOfTextures
	xɎgpAsmall VQ`ȊÕeNX`̑w肵܂B
	KMDWORD		dwNumberOf8x8SmallVQ
	xɎgpA8x8 small VQ`̃eNX`̑w肵܂B
	KMDWORD		dwNumberOf8x8SmallVQmm
	xɎgpA8x8 small VQ mipmap `̃eNX`̑w肵܂B
	KMDWORD		dwNumberOf16x16SmallVQ
	xɎgpA16x16 small VQ`̃eNX`̑w肵܂B
	KMDWORD		dwNumberOf16x16SmallVQmm
	xɎgpA16x16 small VQ mipmap `̃eNX`̑w肵܂B
	KMDWORD		dwNumberOf32x32SmallVQ
	xɎgpA32x32 small VQ`̃eNX`̑w肵܂B
	KMDWORD		dwNumberOf32x32SmallVQmm
	xɎgpA32x32 small VQ mipmap `̃eNX`̑w肵܂B
	KMDWORD		dwNumberOf64x64SmallVQ
	xɎgpA64x64 small VQ`̃eNX`̑w肵܂B
	KMDWORD		dwNumberOf64x64SmallVQmm
	xɎgpA64x64 small VQ mipmap `̃eNX`̑w肵܂B

	8x8 small VQ, 8x8 small VQ mipmap, 64x64 small VQ mipmap݂͌Kamui
	ł͔ΉłB̃oɂ́AɃ[w肵ĂB

Result:
	KMSTATUS_SUCCESS					

--*/

KMSTATUS KMAPI
kmuCalculateKamuiWorkareaSize(
		IN OUT PKMWORKAREASIZE	pWorkareaSize
		);

/*++
kmuCreateTextureSurfaceFromFixed

proto type:

	KMSTATUS
	kmuCreateTextureSurfaceFromFixed(
		IN OUT	PKMDWORD		*pFixedArea,
		OUT		PKMSURFACEDESC	pSurfaceDesc,
		IN		KMINT32			nWidth,
		IN		KMINT32			nHeight,
		IN		KMTEXTURETYPE	nTextureType
		);

Description:
	kmCreateFixedTextureAreaŊmۂŒeNX`̈𕡐
	Ďgpꍇ̃T[tFXfBXNv^𐶐܂B

	*pFixedAreaɎw肳ꂽAhXAŎw肳ꂽ`
	eNX`T[tFXmۂ邱Ƃz肵āAT[tFX
	fBXNv^\̂̓eZbg܂B
	܂A*pFixedAreaɂ́AŎw肳ꂽeNX`̗̈TCY
	Z܂B

	̊֐͂܂ŃT[tFXfBXNv^\̂̓eZbg
	邾łBŒ̈̕Ǘs킯ł͂܂B
	Œ̈̕ǗAceʊǗAAhX̃ACgǗ
	̓[U[ōsĂB

	̊֐́A"small VQk`"A"small VQk`Mipmap"
	ɂ͑ΉĂ܂B̃eNX`Œ̈Ŏgp
	ꍇɂ́AT[tFXfBXNv^̐[U[ōs
	ĂB

Arguments:
	IN OUT	*pFixedArea
		kmCreateFixedTextureAreaŊmۂŒeNX`̈̐擪
		AhXw肵܂Bɂ́Aw肳ꂽeNX`ۑ
		̂ɕKvȗeʂZ܂B
	OUT	pSurfaceDesc(o)
		KMSURFACEDESC^̍\̂ւ̃|C^łB̍\̂ɂ́A
		_ЂɂSurface񂪕Ԃ܂B 
	IN	nWidth, nHeight()
		eNX`̉TCYAcTCYw肵܂B 
		MIPMAPgpꍇɂ̓gbvx̃eNX`TCYw肵܂B
		nWidth, nHeightɎw肷l,8, 16, 32, 64, 128, 256, 512, 1024
		ꂩłȂ΂Ȃ܂B 
	IN	nTextureType()
		eNX`̌`w肵܂B
		"small VQk`"A"small VQk`Mipmap" 
		ꂩ̌`̃eNX`T[tFXw肷邱Ƃł܂B
		kmCreateTextureSurface̐B

Result:
	KMSTATUS_SUCCESS					
	KMSTATUS_INVALID_TEXTURE_TYPE		ȃeNX`^CvATCYw肵
	KMSTATUS_INVALID_ADDRESS			pFixedArea, pSurfaceDesc NULLłB

--*/

KMSTATUS KMAPI
kmuCreateTextureSurfaceFromFixed(
	IN OUT	PKMDWORD		*pFixedArea,
	OUT		PKMSURFACEDESC	pSurfaceDesc,
	IN		KMINT32			nWidth,
	IN		KMINT32			nHeight,
	IN		KMTEXTURETYPE	nTextureType
	);

/*++
kmuGeneratePassTable

proto type:

	KMUPASSSTATUS
	kmuGeneratePassTable(
			IN	PKMVERTEXCONTEXT	pVertexContext,
			IN	KMUINT32			nNumContext,
			OUT	PPKMVERTEXCONTEXT	ppVertexContextTable,
			OUT	PKMUINT32			pPass
		);

Description:
	ݒ肳ꂽ_Ow(pVertexContext)ƂɁA}`pX(Trilinear)̊ePass
	KvȃReLXg𐶐܂B
	  pVertexContextTrilinear1PassڂKMVERTEXCONTEXT͂ƁA̐ݒl
	ePassŕKvKMVERTEXCONTEXT𐶐܂(Opaque:2Pass,Trans:3Pass)
	  TransPolygonTrilineaȑꍇ3PassڂBlendingMode̐ݒ͔Cӂɐݒ\łA
	{֐gpꍇɂ́AL̐ݒɂȂ܂B
	(TransPolygongpꍇ3Pass)
		SRCBlendingMode	:: KM_SRCALPHA
		DSTBlendingMode	:: KM_INVSRCALPHA
	ppVertexContextTableNULLw肳ꂽꍇpPass֕KvȃpX݂̂ԋp܂B

Arguments:
	IN		pVertexContext			_Ow肷KMVERTEXCONTEXTւ̃|C^
									ݒ肵܂B
	IN		nNumContext				ppVertexContextTablẽGg(pX)ݒ肵܂B
									ꂪۂɕKvȃpX菬ꍇ́A
									KMU_PASS_ERROR_VERTEXCONTEXT_PASSԂA
									pPass֕KvȃpX݂̂ԂďI܂B
	OUT		ppVertexContextTable	ꂽ}`pXpKMVERTEXCONTEXT󂯎邽߂́A
									KMVERTEXCONTEXTւ̃|C^̔zւ̃|C^ݒ肵܂B
									NULLw肳ꂽꍇpPass֕KvȃpX݂̂Ԃ܂B
	OUT		pPass					ݒ肳ꂽ_OwŕKvƂȂ}`pXԂ܂B

Result:
	KMU_PASS_OK							ݒ萬
	KMU_PASS_ERROR_NOT_IMPLEMENTED		݂̃VXeł́A{֐͑ΉĂ܂
	KMU_PASS_ERROR_VERTEXCONTEXT		pVertexContexts(NULL)ł
	KMU_PASS_ERROR_VERTEXCONTEXT_PASS	w肵pX(nNumContext)ł͕sĂ܂
--*/

KMUPASSSTATUS KMAPI
kmuGeneratePassTable(
		IN	PKMVERTEXCONTEXT	pVertexContext,
		IN	KMUINT32			nNumContext,
		OUT	PPKMVERTEXCONTEXT	ppVertexContextTable,
		OUT	PKMUINT32			pPass
	);


/*++
kmuCheckPassTable

proto type:

	KMUPASSSTATUS
	kmuCheckPassTable(
			IN	PPKMVERTEXCONTEXT	ppVertexContextTable,
			IN	KMUINT32			nNumContext,
			OUT	PKMUINT32			pPass
		);

Description:
	w肳ꂽKMVERTEXCONTEXTe[ůeReLXg̓eۂ`FbN܂B

Arguments:
	IN		ppVertexContextTable	KMVERTEXCONTEXTւ̃|C^̔zւ̃|C^ݒ肵܂B
	IN		nNumContext				ppVertexContextTablẽGgݒ肵܂B
	OUT		pPass					G[ꍇɂ́AԈĂKMVERTEXCONTENT
									GgԍZbg܂B

Result:
	KMU_PASS_OK								ݒ萬
	KMU_PASS_ERROR_VERTEXCONTEXT			KMVERTEXCONTEXTs(NULL)ł
	KMU_PASS_ERROR_VERTEXCONTEXT_PASS		nNumContexts(1)ł
	KMU_PASS_ERROR_PARAMTYPE				KMVERTEXCONTEXT ParamTypesł
	KMU_PASS_ERROR_LISTTYPE					KMVERTEXCONTEXT ListTypesł
	KMU_PASS_ERROR_MIPMAP_D_ADJUST			KMVERTEXCONTEXT MipMapAdjustsł
	KMU_PASS_ERROR_FOGMODE					KMVERTEXCONTEXT FogModesł
	KMU_PASS_ERROR_FILTERMODE				KMVERTEXCONTEXT FilterModesł
	KMU_PASS_ERROR_TEXTURESHADINGMODE		KMVERTEXCONTEXT TextureShadingModesł
	KMU_PASS_ERROR_COLORTYPE				KMVERTEXCONTEXT ColorTypesł
	KMU_PASS_ERROR_SHADINGMODE				KMVERTEXCONTEXT ShadingModesł
	KMU_PASS_ERROR_USERCLIPMODE				KMVERTEXCONTEXT UserClipModesł
	KMU_PASS_ERROR_TRILINEAR_SETTING		Trilinearݒ̑gݍ킹sł
											 KMVERTEXCONTEXT ListType/ShadingMode/FilterMode
											SRCBlendingMode/DSTBlendingMode/bSRCSel/bSRCSel
											̂ꂩATrilinearŎgpłȂݒłB
	KMU_PASS_ERROR_SPRITE_SETTING			Spriteݒ̑gݍ킹sł
											 KMVERTEXCONTEXT ShadingMode/ColorType/UVFormat
											̂ꂩASpriteŎgpłȂݒłB
	KMU_PASS_ERROR_BLENDINGMODE_SETTING		BlendingModeݒ̑gݍ킹sł
											 OpaquePolygongp(ListType:KM_OPAQUE_POLYGON)
											KMVERTEXCONTEXT SRCBlendingMode/DSTBlendingMode
											ݒs̑gݍ킹w肳ꂽꍇł
											(SRCBlendingMode:KM_ONE , DSTBlendingMode:KM_ZEROȊO)
	KMU_PASS_ERROR_MODIFIER_SETTING			Modifierݒ̑gݍ킹sł
											 Modifiergp(ParamType:KM_MODIFIERVOLUME)
											KMVERTEXCONTEXT ModifierInstructionsȏꍇłB

--*/

KMUPASSSTATUS KMAPI
kmuCheckPassTable(
		IN	PPKMVERTEXCONTEXT	ppVertexContextTable,
		IN	KMUINT32			nNumContext,
		OUT	PKMUINT32			pPass
	);


/*++
kmuConvertStripContext

proto type:

	KMSTATUS
	kmuConvertStripContext(
		OUT	PKMVOID				pStripContext,
		IN	PKMVERTEXCONTEXT	pVertexContext1,
		IN	PKMVERTEXCONTEXT	pVertexContext2
	)

Description:
	]݊KMVERTEXCONTEXT\̂AKAMUI2KMSTRIPCONTEXT(KMTWOVOLUMESTRIPCONTEXT)
	ւ̓ݒl̃Ro[gȂ܂B

Arguments:
	OUT		pStripContext			KMSTRIPCONTEXT܂́AKMTWOVOLUMESTRIPCONTEXT\̂ւ̃|C^
									w肵܂BParam2gpꍇ́AKMTWOVOLUMESTRIPCONTEXT
									̃|C^gpĂB
	IN		pVertexContext1			Param1ŎgpKMVERTEXCONTEXT\̂ւ̃|C^w肵܂B
	IN		pVertexContext2			Param2ŎgpKMVERTEXCONTEXT\̂ւ̃|C^w肵܂B
									Param2gpȂꍇ́ANULLw肵ĂB

Result:
	KMSTATUS_SUCCESS				ݒ萬
	KMSTATUS_INVALID_ADDRESS		s(NULL)łB
	KMSTATUS_INVALID_SETTING		pStripContext̃TCYݒ肪słB
--*/

KMSTATUS KMAPI
kmuConvertStripContext(
		OUT	PKMVOID				pStripContext,
		IN	PKMVERTEXCONTEXT	pVertexContext1,
		IN	PKMVERTEXCONTEXT	pVertexContext2
	);

#ifdef __cplusplus
}
#endif 

#endif
