/*
	Ninja Library

	COPYRIGHT (C) SEGA ENTERPRISES,LTD.

*/
#ifndef	_NU_MOTION_H_
#define	_NU_MOTION_H_

/*--------------------------------------*/
/*      Motion define                   */
/*--------------------------------------*/
/* motion type bits */

#define NUD_MTYPE_POS_0			BIT_0
#define NUD_MTYPE_ANG_1			BIT_1
#define NUD_MTYPE_SCL_2			BIT_2
#define NUD_MTYPE_VEC_3			BIT_3
#define NUD_MTYPE_VEC_0			BIT_4
#define NUD_MTYPE_SANG_1		BIT_5
#define	NUD_MTYPE_TARGET_3		BIT_6
#define	NUD_MTYPE_ROLL_6		BIT_7
#define	NUD_MTYPE_ANGLE_7		BIT_8
#define	NUD_MTYPE_RGB_8			BIT_9
#define	NUD_MTYPE_INTENSITY_9	BIT_10
#define	NUD_MTYPE_POINT_9		BIT_12
#define NUD_MTYPE_QUAT_1        BIT_13
#define NUD_MTYPE_SHAPEID		BIT_14    /* compact shape            */
#define NUD_MTYPE_EVENT_4		BIT_15

#define NUD_MTYPE_LINER         0x0000    /* use liner                */
#define NUD_MTYPE_SPLINE        0x0040    /* use spline               */
#define NUD_MTYPE_USER          0x0080    /* use user function        */
#define NUD_MTYPE_MASK          0x00c0    /* motion function mask     */



/*	Rotate Type of the return code of nuGetMotionNodeData function */
#define	NUD_MROT_ANGLEXYZ		0
#define	NUD_MROT_ANGLEZXY		1
#define	NUD_MROT_QUATERNION		2

/* Motion event flag */
#define	NUD_EVENT_HIDE			BIT_0

/*
 * NUS_MOTION
*/
/* key frame animation */
typedef struct {
	Uint32          keyframe;
	Float           key[3];
} NUS_MKEY_F;

typedef struct {
	Uint32          keyframe;
	Angle           key[3];     /* angle                        */
} NUS_MKEY_A;

typedef struct {
	Uint16          keyframe;
	Sangle          key[3];     /* angle                        */
} NUS_MKEY_SA;

typedef struct {
	Uint32          keyframe;
	Angle           angle;      /* angle                        */
	Float           axe[3];     /* axe vector                   */
} NUS_MKEY_AX;

typedef struct
{
	Uint32          keyframe;
	void            *key;        /* pointer                     */
} NUS_MKEY_P;

typedef struct {
	Uint32          keyframe;
	Float           key;
} NUS_MKEY_F1;

typedef struct {
	Uint32          keyframe;
	Float           key[2];
} NUS_MKEY_F2;

typedef struct {
	Uint32          keyframe;
	Float           nrang;
	Float           frang;
	Angle           iang;
	Angle           oang;
} NUS_MKEY_SPOT;

typedef struct {
	Uint32          keyframe;
	Uint32          key;
} NUS_MKEY_UI32;

typedef struct {
	Uint32          keyframe;
	Sint32          key;
} NUS_MKEY_SI32, NUS_MKEY_A1;

typedef struct {
    Uint32          keyframe;
    Float           key[4];
} NUS_MKEY_QUAT;

typedef struct {
	Uint32          keyframe;
	Uint32          shapeId;
} NUS_MKEY_SHAPEID;

typedef struct {
	Uint16          keyframe;
	Sangle          key;
} NUS_MKEY_SA1;

typedef struct {
	Uint16          keyframe;
	Uint16          key;
} NUS_MKEY_UI16;

typedef struct {
	NUS_MKEY_SHAPEID    *keys;
	Uint16              nbKey;
	Uint16              entryId;
} NUS_CSHAPEDATA;

/*
 * NUS_MDATA
 */
/* 1 factor : pos or ang or scl or shape or vec */
typedef struct {
	void            *p[1];      /* factor pointer               */
	Uint32          nb[1];      /* factor count                 */
} NUS_MDATA1;

/* 2 factors : pos and ang, pos and vec(light motion), etc      */
typedef struct {
	void            *p[2];      /* factor pointer               */
	Uint32          nb[2];      /* factor count                 */
} NUS_MDATA2;

/* 3 factors : pos and ang and scl, pos and ang and shape, etc  */
typedef struct {
	void            *p[3];      /* factor pointer               */
	Uint32          nb[3];      /* factor count                 */
} NUS_MDATA3;

/* 4 factors : pos and ang and scl and shape, etc               */
typedef struct {
	void            *p[4];      /* factor pointer               */
	Uint32          nb[4];      /* factor count                 */
} NUS_MDATA4;

/* 5 factors : etc               */
typedef struct {
	void            *p[5];      /* factor pointer               */
	Uint32          nb[5];      /* factor count                 */
} NUS_MDATA5;

typedef struct {
	void            *mdata;     /* NUS_MDATA array              */
	Uint32          nbFrame;    /* frame count                  */
	Uint16          type;       /* motion type  NUD_MTYPE_...   */
	Uint16          inp_fn;     /* interpolation & factor count */
} NUS_MOTION;

typedef struct
{
	NUS_MOTION		*motion[2];		/* motion's pointers */
	Float			frame[2];		/* current frames */
} NUS_MOTION_LINK;


typedef struct {
	NJS_CNK_OBJECT  *object;    /* Object Tree top pointer      */
	NUS_MOTION      *motion;    /* motion                       */
#if 0
	NUS_TEXLIST     *texlist;   /* current texture list         */
#endif
} NUS_ACTION;

typedef struct
{
	NJS_CNK_OBJECT  *object;		/* object's pointer */
	NUS_MOTION_LINK	*motionlink;	/* motion-link's pointer */
} NUS_ACTION_LINK;


/*--------------------------------------*/
/*   Compact shape define               */
/*--------------------------------------*/

/*
 * NUS_CSHAPE
 */
/* compact shape animation */
typedef struct {
	Sint32          *cv;        /* chunk vertex pointer         */
} NUS_CSHAPECV;

typedef struct {
	NUS_CSHAPECV    *cvlist;    /* key frame group chunk vertex data */
	Uint16          groupId;    /* key frame group Id           */
	Uint16          reserved;   /* reserved zone                */
} NUS_CSHAPE_KEYGROUP;

typedef struct {
	NUS_CSHAPE_KEYGROUP *shapes;    /* key frame groups         */
	Uint32              nbShape;    /* key frame group count    */
} NUS_CSHAPELIST;


/*--------------------------------------*/
/*   Light & Camera Motion Structure    */
/*--------------------------------------*/

typedef struct {
	NJS_LIGHT	*light;
	NUS_MOTION	*motion;
} NUS_LACTION;

typedef struct {
	NJS_CAMERA	*camera;
	NUS_MOTION	*motion;
} NUS_CACTION;

typedef struct {
	Float	*pos;
	Angle	*ang;
	Float	*scl;
} NUS_MOTION_DATA;

typedef struct {
	Float	*pos;
	Float	*vect;
	Sangle	*roll;
	Sangle	*ang;
} NUS_CMOTION_DATA;

typedef struct {
	Float	*pos;
	Float	*rgb;
	Float	*rang;
} NUS_LMOTION_DATA;

typedef struct{
	Int 		n;
	NJS_LIGHT	**lights;
	NUS_MOTION	**motions;
}NUS_MLIGHT_MOTION;
/*		prototype declarations			*/

/*
	High level motion functions
*/

/*	set shape buffer	*/
void	nuInitShape( void *buf, Sint32 size );
void	nuSetShapeList( const NUS_CSHAPELIST *shape_list );
void	nuSetMotionCallback( void (*func)(NJS_CNK_OBJECT *obj) );
void	nuInitShapeLink( void *buf, Sint32 size );
void	nuSetShapeLinkList( const NUS_CSHAPELIST *shape_list );

/*	draw motion */
void	nuCnkEasyDrawMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, Float frame );
void	nuCnkSimpleDrawMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, Float frame );
void	nuCnkEasyMultiDrawMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, Float frame );
void	nuCnkSimpleMultiDrawMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, Float frame );

/*	draw shape motion */
void	nuCnkEasyDrawShapeMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, const NUS_MOTION *shape, Float frame );
void	nuCnkSimpleDrawShapeMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, const NUS_MOTION *shape, Float frame );
void	nuCnkEasyMultiDrawShapeMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, const NUS_MOTION *shape, Float frame );
void	nuCnkSimpleMultiDrawShapeMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, const NUS_MOTION *shape, Float frame );

/*	draw motion link	*/
void	nuCnkEasyDrawMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, Float rate );
void	nuCnkSimpleDrawMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, Float rete );
void	nuCnkEasyMultiDrawMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, Float rate );
void	nuCnkSimpleMultiDrawMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, Float rate );

/*	draw shape motion link		*/
void	nuCnkEasyDrawShapeMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, const NUS_MOTION_LINK *shapelink, Float rate );
void	nuCnkSimpleDrawShapeMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, const NUS_MOTION_LINK *shapelink, Float rate );
void	nuCnkEasyMultiDrawShapeMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, const NUS_MOTION_LINK *shapelink, Float rate );
void	nuCnkSimpleMultiDrawShapeMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motionlink, const NUS_MOTION_LINK *shapelink, Float rate );

/*	get draw motion	*/
void	nuGetDrawMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, NUS_MOTION_DATA *data, Float frame );

/*	camera motion	*/
void	nuCameraAction( const NUS_CACTION *caction, Float frame );
void	nuCameraMotion( const NJS_CAMERA *camera, const NUS_MOTION *motion, Float frame );

/*	get camera motion	*/
void	nuGetCameraAction( const NUS_CACTION *caction, NUS_CMOTION_DATA *data, Float frame );
void	nuGetCameraMotion( const NJS_CAMERA *camera, const NUS_MOTION *motion, NUS_CMOTION_DATA *data, Float frame );

/*	light motion	*/
void	nuLightAction( Int light_num, const NUS_LACTION *laction, Float frame );
void	nuLightMotion( Int light_num, const NJS_LIGHT *light, const NUS_MOTION *motion, Float frame );

/*	get light motion	*/
void	nuGetLightAction( const NUS_LACTION *laction, NUS_LMOTION_DATA *data, Float frame );
void	nuGetLightMotion( const NJS_LIGHT *light, const NUS_MOTION *motion, NUS_LMOTION_DATA *data, Float frame );

/* multilight motion */
void	nuMultiLightMotion(const NUS_MLIGHT_MOTION *mlight,Float frame);

/*
	Low level motion functions
*/

//	motion core
void	nuStartMotionEx( const NUS_MOTION *motion, Float frame );
void	nuSetNextMotionNodeEx( void );
void	nuSetCurMotionInfoSlot( int Slot );

//	object motion
void	nuStartMotionObj( const NUS_MOTION *motion, Float frame );
void	nuStartMotionObjEx( const NUS_MOTION *motion, Float frame );
int		nuGetMotionNodeData( const NJS_CNK_OBJECT *cnkobj, Float pos[3], Angle ang[3], Float scl[3], NJS_QUATERNION *qua );
void	nuMotionTransformEx( const NJS_CNK_OBJECT *cnkobj );
void	nuChangeMotionObject( NJS_CNK_OBJECT *cnkobj);
Sint32	nuGetMotionEventFlag( void );

//	motion link
void	nuStartMotionLink( const NUS_MOTION_LINK *motion_link, Float rate );
int		nuGetMotionLinkNodeData( const NJS_CNK_OBJECT *cnkobj, Float pos[3], Angle ang[3], Float scl[3], NJS_QUATERNION *qua );
void	nuMotionLinkTransformEx( const NJS_CNK_OBJECT *cnkobj );
void	nuSetNextMotionLinkNode( void );

//	shape motion
void	nuSetCurShapeInfoSlot( int Slot );
void	nuStartShapeEx( const NUS_MOTION *shape, Float frame );
void	nuSetNextShapeNodeEx( void );
void	nuCalcShapeVlist( const NJS_CNK_MODEL *cnkmodel );
void	nuGetShapeGroupID( void );
void	nuSetShapeVlist( Uint32 *vlist, Uint32 vtype );

//	shape link
void	nuStartShapeLink( const NUS_MOTION_LINK *shape_link, Float rate );
void	nuSetNextShapeLinkNode( void );
void	nuCalcShapeLinkVlist( const NJS_CNK_MODEL *cnkmodel );




//	Motion, Shape, MotionLink, ShapeLink with a specified function.
void	nuDrawMotion( const NJS_CNK_OBJECT *object, const NUS_MOTION *motion, Float frame,
					   Sint32 ( *pDrawFunc )( NJS_CNK_MODEL *cnkmodel ) );

void	nuDrawMotionLink( const NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motion_link,
						   Float rate, Sint32 ( *pDrawFunc )( NJS_CNK_MODEL *cnkmodel ) );

void	nuCnkDrawShapeMotion( NJS_CNK_OBJECT *object, const NUS_MOTION *motion, const NUS_MOTION *shape, Float frame, Float sframe,
							   Sint32 ( *pDrawFunc )( NJS_CNK_MODEL *cnkmodel ) );

void	nuCnkDrawShapeLink( NJS_CNK_OBJECT *object, const NUS_MOTION_LINK *motion_link, const NUS_MOTION_LINK *shape_link,
							 Float rate, Sint32 ( *pDrawFunc )( NJS_CNK_MODEL *cnkmodel ) );


#endif	/*_NU_MOTION_H_*/
