/*--------------------------------------------------------------------------------
	HEAP LIBRARY
--------------------------------------------------------------------------------*/

#ifndef _NWHEAP_H_
#define _NWHEAP_H_

extern "C" {

#include <stdio.h>
#include <string.h>
#include <shinobi.h>

/*------------------------------------------------------------------------------*/

#define HL_ALIGN			(0x20)				/* ACTCY			*/
#define HL_ALIGN_N			(0xffffffe0)		/* ACTCY			*/
#define HL_HEADER_SIZE		(0x20)				/* wb_̃TCY			*/

#define HL_ERROR			(0x00000000)
#define HL_USED				(0x00000001)
#define HL_TOPOFLIST		(0x00000002)
#define HL_ENDOFLIST		(0x00000003)

typedef enum {
	HL_ERROR_INIT	= 0	,						/* ĂȂ		*/
	HL_ERROR_DUP_INIT	,						/* dɏ			*/
	HL_ERROR_LEAK		,						/* [N				*/
	HL_ERROR_BREAK		,						/* j󂳂Ă	*/
	HL_ERROR_MEMORY		,						/* s				*/
	HL_ERROR_IVALID		,						/* sȃp[^			*/
	HL_ERROR_POINTER	,						/* |C^s			*/
	HL_ERROR_NOBEGIN	,						/* BEGINĂȂ̂END		*/
	HL_ERROR_MISMATCH	,						/* MallocFree񐔂̕sv	*/
	HL_WARNING_COUNT	,						/* mۂƊJ񐔂sv	*/
	HL_WARNING_HOLE		,						/* z[		*/
	HL_BREAK_POINT		 						/* u[N|Cg			*/
} HLSTATUS;

typedef enum {
	HL_INIT			= 0	,
	HL_EXIT				,
	HL_MALLOCSTAT		,
	HL_MALLOC			,
	HL_CALLOC			,
	HL_REALLOC			,
	HL_FREE				,
	HL_BEGINCHECK		,
	HL_ENDCHECK			,
	HL_DUMP
} HLCOMMAND;

/*------------------------------------------------------------------------------*/

typedef struct _TAG_HEADER_ {
	unsigned long		nFlag;					/* tO					*/
	unsigned long		nSize;					/* ubNTCY(wb_)	*/
	struct _TAG_HEADER_	*pNext;					/* |C^				*/
	struct _TAG_HEADER_	*pBack;					/* O|C^				*/
	unsigned long		nCount;					/* Check̃JEg			*/
	unsigned long		nAlloc;					/* Malloc̃JEg			*/
	unsigned long		nFree;					/* FreẽJEg			*/
	unsigned long		PR;						/* PRWX^(CALL)		*/
} *HEAPHANDLE;

typedef void (*HLCALLBACKFUNC)( HLCOMMAND eComd, HLSTATUS eStat, HEAPHANDLE pHandle );

typedef struct _TAG_GLOBAL_ {
	unsigned long		nSize;					/* \̂̃TCY			*/
	unsigned long		nStat;					/* Xe[^X				*/
	unsigned long		nCount;					/* `FbNJE^[		*/
	unsigned long		nAlloc;					/* Malloc			*/
	unsigned long		nFree;					/* Free				*/
	HLCALLBACKFUNC		pFunc;					/* R[obNAhX		*/
	HEAPHANDLE			pHandle;				/* nh			*/
	unsigned long		pad;
} HEAPGLOBAL;

/*------------------------------------------------------------------------------*/

void hlHeapInit( void *pHeapStart, unsigned long nHeapSize, HLCALLBACKFUNC pFunc );
void hlHeapExit( void );
void hlMallocStat( unsigned long *nFreeSize, unsigned long *nBiggestSize );

void *hlMalloc( unsigned long nSize );
void *hlMalloc_( unsigned long nSize );
void hlFree( void *pMem );
void *hlRealloc( void *op, unsigned long nSize );
void *hlRealloc_( void *op, unsigned long nSize );
void *hlCalloc( size_t nObj, size_t nSize );
void *hlCalloc_( size_t nObj, size_t nSize );

void hlBeginCheck( void );
void hlEndCheck( void );
void hlHeapDump( void );

char *hlComdToStr( HLCOMMAND eComd );
char *hlStatToStr( HLSTATUS eStat );

static void* operator new( size_t size )
{
	return hlMalloc( size );
}
static void operator delete( void *p )
{
	hlFree( p );
}
/*------------------------------------------------------------------------------*/

}
#endif
