#include <shinobi.h>
#include <Ninja.h>

#include "njuTask.h"

typedef struct {
	NJS_TASK	*task_top;
	NJS_TASK	*task_bottom;
} NJS_TASK_PRI;

static	NJS_TASK_PRI	*task_pri;
static	Int				task_pri_cnt;

Int		_nj_task_cnt_;

/****************************************************************/
/*  ֐Fhs								*/
/*  @  \F^XN̏		                            */
/*    ́FInt	n:^XNvCIeB                        */
/*  o  ́FȂ                                                */
/*  ߂lFȂ                                                */
/*  T@vF                                                    */
/*  @lF                                                    */
/*                                                              */
/****************************************************************/
void njuInitTask( Int n )
{
	Int	i;

	task_pri = (NJS_TASK_PRI*)syMalloc( sizeof(NJS_TASK_PRI)*n );
	if( task_pri == NULL ) return;

	for( i=0; i<n; i++ ) {
		(task_pri+i)->task_top = NULL;
		(task_pri+i)->task_bottom = NULL;
	}
	task_pri_cnt = n;

	_nj_task_cnt_ = 0;
}

/****************************************************************/
/*  ֐Fbs							*/
/*  @  \F^XN̐			                            */
/*    ́Ffunc	:^XN֐                                 */
/*			name	:^XN̖O                               */
/*			pri		:^XN̗Dx                             */
/*			size	:^XÑobt@TCY                     */
/*			buf		:obt@                             */
/*			wktask	:^XNʐMp|C^                       */
/*  o  ́FȂ                                                */
/*  ߂lF^XN|C^                                      */
/*  T@vF                                                    */
/*  @lF                                                    */
/*                                                              */
/****************************************************************/
NJS_TASK *njuCreateTask( void (*func)( NJS_TASK * ), Int name, Int pri, Int size, void *buf, NJS_TASK *wktask )
{
	NJS_TASK	*task;
	NJS_TASK	*task_top;
	NJS_TASK	*task_bottom;
	void		*work;

	if( pri >= task_pri_cnt ) return NULL;

	task = (NJS_TASK*)syMalloc( sizeof(NJS_TASK) );
	if( task == NULL ) return NULL;

	if( size != 0 ) {
		work = syMalloc( size );
		if( work == NULL ) {
			syFree( task );
			return NULL;
		}
	}
	else {
		work = NULL;
	}
	
	task_top = (task_pri+pri)->task_top;
	task_bottom = (task_pri+pri)->task_bottom;

	task->status = 0;
	task->func = func;
	task->buf = work;
	task->task = wktask;
	task->pri = pri;
	task->name = name;

	if( task_top == NULL ) {		/* ŏ̃^XN	*/
		task_top = task;
		task->next = NULL;
		task->prev = NULL;
		task_bottom = task;
	}
	else {							/* Ōɒǉ	*/
		task->next = NULL;
		task->prev = task_bottom;
		task_bottom->next = task;
		task_bottom = task;
	}

	(task_pri+pri)->task_top = task_top;
	(task_pri+pri)->task_bottom = task_bottom;
	
	if( (buf != NULL) && (work != NULL) ) {
		syMemCopy( work, buf, size );
	}
	
	(*func)( task );
	
	_nj_task_cnt_ ++;

	return task;
}

/****************************************************************/
/*  ֐F													*/
/*  @  \F						                            */
/*    ́F                                                    */
/*  o  ́FȂ                                                */
/*  ߂lFȂ                                                */
/*  T@vF                                                    */
/*  @lF                                                    */
/*                                                              */
/****************************************************************/
void njuDeleteTask( NJS_TASK *task )
{
	NJS_TASK	*task_top;
	NJS_TASK	*task_bottom;
	Int			pri;

	pri = task->pri;
	task_top = (task_pri+pri)->task_top;
	task_bottom = (task_pri+pri)->task_bottom;

	if( task->buf != NULL ) {
		syFree( task->buf );
	}
	
	if( task_top == task_bottom ) {		/* Ō̂P	*/
		task_top = NULL;
		task_bottom = NULL;
	}
	else if( task == task_top ) {		/* gbṽ^XN폜ꂽ	*/
		task_top = task->next;
		task_top->prev = NULL;
	}
	else if( task == task_bottom ) {	/* {g̃^XN폜ꂽ	*/
		task_bottom = task->prev;
		task_bottom->next = NULL;
	}
	else {								/* r̃^XN폜ꂽ		*/
		task->prev->next = task->next;
		task->next->prev = task->prev;
	}
	
	(task_pri+pri)->task_top = task_top;
	(task_pri+pri)->task_bottom = task_bottom;

	syFree( task );

	_nj_task_cnt_ --;
}

/****************************************************************/
/*  ֐F													*/
/*  @  \F						                            */
/*    ́FȂ                                                */
/*  o  ́FȂ                                                */
/*  ߂lFȂ                                                */
/*  T@vF                                                    */
/*  @lF                                                    */
/*                                                              */
/****************************************************************/
void njuExecTask()
{
	Int		i;
	NJS_TASK	*task,*next;
	
	for( i=0; i<task_pri_cnt; i++ ) {
		task = 	(task_pri+i)->task_top;
		while( task != NULL ) {
			next = task->next;			/* ֐Ń^XN邩Ȃ̂ */
			(*task->func)(task);
			task = next;
		}
	}
}

/****************************************************************/
/*  ֐F													*/
/*  @  \F						                            */
/*    ́FȂ                                                */
/*  o  ́FȂ                                                */
/*  ߂lFȂ                                                */
/*  T@vF                                                    */
/*  @lF                                                    */
/*                                                              */
/****************************************************************/
void njuExitTask()
{
	Int		i;
	NJS_TASK	*task,*next;
	
	for( i=0; i<task_pri_cnt; i++ ) {
		task = 	(task_pri+i)->task_top;
		while( task != NULL ) {
			next = task->next;
			njuDeleteTask(task);
			task = next;
		}
	}
	syFree( task_pri );
}

static	NJS_TASK	*task_search_p;
static	Int			task_search_name;

/****************************************************************/
/*  ֐F													*/
/*  @  \F						                            */
/*    ́F                                                    */
/*  o  ́FȂ                                                */
/*  ߂lF                                                    */
/*  T@vF                                                    */
/*  @lF                                                    */
/*                                                              */
/****************************************************************/
NJS_TASK *njuSearchTask( Int name, Int pri )
{
	NJS_TASK	*task;

	task = 	(task_pri+pri)->task_top;

	task_search_p = task;
	task_search_name = name;

	while( task != NULL ) {
		if( task->name == name ) {
			task_search_p = task->next;
			break;
		}
		task = task->next;
	}

	return task;
}

/****************************************************************/
/*  ֐F													*/
/*  @  \F						                            */
/*    ́FȂ                                                */
/*  o  ́FȂ                                                */
/*  ߂lF                                                    */
/*  T@vF                                                    */
/*  @lF                                                    */
/*                                                              */
/****************************************************************/
NJS_TASK *njuSearchTaskNext( void )
{
	NJS_TASK	*task;
	Int			name;

	task = 	task_search_p;
	name = task_search_name;

	while( task != NULL ) {
		if( task->name == name ) {
			task_search_p = task->next;
			break;
		}
		task = task->next;
	}

	return task;
}
