/*---------------------------------------------------------------------------
  Shinobi2 Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.
---------------------------------------------------------------------------*/

#include	<shinobi.h>
#include	"Nj.h"

#define INTERVAL  (500)
#define STACKSIZE (0x10000)

static SYFBR hFbr[4];
static Uint32 Stack[3][ STACKSIZE / sizeof(Uint32) ];

volatile Uint32 CurrentFbr;
volatile Sint32 SORT[3][256];
volatile Sint32 StartFlag[3];

void TimerFunc( void *p );
void ThreadFuncA( void *p );
void ThreadFuncB( void *p );
void ThreadFuncC( void *p );

/*---------------------------------------------------------------------------
		
		initialize
---------------------------------------------------------------------------*/
Sint32	UserInit(void)
{
	hFbr[0] = syFbrGetMainFiber();
	hFbr[1] = syFbrCreateFiber( (void*)&Stack[0][0], STACKSIZE, ThreadFuncA, NULL );
	hFbr[2] = syFbrCreateFiber( (void*)&Stack[1][0], STACKSIZE, ThreadFuncB, NULL );
	hFbr[3] = syFbrCreateFiber( (void*)&Stack[2][0], STACKSIZE, ThreadFuncC, NULL );
	
	CurrentFbr   = 0;
	StartFlag[0] = 0;
	StartFlag[1] = 0;
	StartFlag[2] = 0;
	
	syTmrGenSetClock( SYD_TMR_CLOCK_P64 );
	syTmrGenSetCount( -syTmrGenMicroToCount( INTERVAL, SYD_TMR_CLOCK_P64 ) );
	syTmrGenSetInt( TimerFunc, NULL );
	syTmrGenStart();
	
	return USER_CONTINUE;
}
/*---------------------------------------------------------------------------
		I
		exit
---------------------------------------------------------------------------*/
Sint32	UserExit(void)
{
	syTmrGenStop();
	syFbrDeleteFiber( hFbr[1] );
	syFbrDeleteFiber( hFbr[2] );
	syFbrDeleteFiber( hFbr[3] );
	
	return USER_CONTINUE;
}
/*---------------------------------------------------------------------------
		C
		main
---------------------------------------------------------------------------*/
Sint32	UserMain(void)
{
	Sint32 i,j;
	NJS_POLYGON_VTX vtx[4];
	
	if (Per->press&PDD_DGT_TA) {
		if (StartFlag[0]+StartFlag[1]+StartFlag[2]==0) {
			for( j=0; j<256; j++ ) {
				SORT[0][j] = (rand()>>8) + 16;
				SORT[1][j] = SORT[0][j];
				SORT[2][j] = SORT[0][j];
			}
			StartFlag[0] = 1;
			StartFlag[1] = 1;
			StartFlag[2] = 1;
		}
	}
	
	vtx[0].col = 0xffffff00;
	vtx[1].col = 0xffffff00;
	vtx[2].col = 0xff00ffff;
	vtx[3].col = 0xff00ffff;
	vtx[0].z   = 1.f;
	vtx[1].z   = 1.f;
	vtx[2].z   = 1.f;
	vtx[3].z   = 1.f;
	
	njDrawPolygon2DExStart( FALSE );
	for( i=0; i<3; i++ ) {
		vtx[0].x = i*192.f + 64.f;
		vtx[1].x = vtx[0].x;
		
		for( j=0; j<256; j++ ) {
			vtx[2].x = vtx[0].x + SORT[i][j];
			vtx[3].x = vtx[2].x;
			vtx[0].y = j + 144.f;
			vtx[1].y = j + 145.f;
			vtx[2].y = vtx[0].y;
			vtx[3].y = vtx[1].y;
			njDrawPolygon2DExSetData( vtx, 4 );
		}
	}
	njDrawPolygon2DExEnd();
	
	njPrintC( NJM_LOCATION( 2,2 ), "PRESS A BUTTON" );
	return USER_CONTINUE;
}
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
void swap( Sint32 *a, Sint32 *b )
{
	Sint32 n = *a;
	*a = *b;
	*b = n;
	
	n = syTmrGetCount();
	while( syTmrCountToMicro( syTmrGetCount()-n )<600 );
}
/*-------------------------------------------------------------------------*/
void TimerFunc( void *p )
{
	syTmrGenSetCount( -syTmrGenMicroToCount( INTERVAL, SYD_TMR_CLOCK_P64 ) );
	syTmrGenSetInt( TimerFunc, NULL );
	syTmrGenStart();
	
	CurrentFbr = (CurrentFbr+1) & 0x3;
	syFbrDelayedSwitchToFiber( hFbr[CurrentFbr] );
}
/*-------------------------------------------------------------------------*/
void ThreadFuncA( void *p )
{
	Sint32 i,j;
	
	while(1) {
		while( StartFlag[0]==0 );
		
		for( i=0; i<255; i++ )
			for( j=i; j<256; j++ )
				if (SORT[0][i]>SORT[0][j])
					swap( &SORT[0][i], &SORT[0][j] );
		
		StartFlag[0] = 0;
	}
}
/*-------------------------------------------------------------------------*/
void block( Sint32 nFirst, Sint32 nNum )
{
	Sint32 i;
	Sint32 n     = nNum>>1;
	Sint32 nLast = nFirst + nNum;
	
	for( i=0; i<n; i++ ) 
		if (SORT[1][i+nFirst]>SORT[1][nLast - i - 1])
			swap( &SORT[1][i+nFirst], &SORT[1][nLast - i -1] );
	
	if (nNum>2) {
		block( nFirst  , n );
		block( nFirst+n, n );
	}
}
void ThreadFuncB( void *p )
{
	Sint32 i,j,n;
	
	while(1) {
		while( StartFlag[1]==0 );
		
		block( 0, 256 );
		
		i = 0;
		while( i<255 ) {
			if (SORT[1][i]>SORT[1][i+1]) {
				swap( &SORT[1][i], &SORT[1][i+1] );
				if (i>0) i--;
			} else {
				i++;
			}
		}
		
		StartFlag[1] = 0;
	}
}
/*-------------------------------------------------------------------------*/
static void quick( Sint32 nFirst, Sint32 nLast )
{
	Sint32 i = nFirst;
	Sint32 j = nLast;
	Sint32 n = SORT[2][nFirst];
	
	while(1) {
		while( SORT[2][i]<n ) i++;
		while( SORT[2][j]>n ) j--;
		if (i >= j) break;
		
		swap( &SORT[2][i], &SORT[2][j] );
		
		i++;
		j--;
	}
	
	if (nFirst < i-1 ) quick( nFirst, i-1 );
	if (nLast  > j+1 ) quick( j+1, nLast );
}

void ThreadFuncC( void *p )
{
	Sint32 nFirst,nLast,n;
	
	while(1) {
		while( StartFlag[2]==0 );
		
		quick( 0, 255 );
		
		StartFlag[2] = 0;
	}
}
/*-------------------------------------------------------------------------*/
