/*
 * fntDisp.c
 *
 * choose font code and display font
 *
 * for shinogi2
 */

#include <shinobi.h>
#include <ninja.h>
#include "fntDisp.h"

#define COLOUR565_WHITE (0xc618)
#define COLOUR565_BLUE  (0x0018)
#define COLOUR565_GREEN (0x0600)
#define COLOUR565_RED   (0xc000)
#define COLOUR565_BLACK (0x0000)

#define BACK_COLOUR565 COLOUR565_WHITE
#define FORE_COLOUR565 COLOUR565_BLACK

NJS_TEXNAME texname[2] ;
NJS_TEXLIST texlist = {texname, 2};
Uint16 kanjiBuff[KANJI_BUFF_W*KANJI_BUFF_H];
Uint16 *kanjiBuffPtr;
NJS_TEXINFO	kanjiInfo[2];
NJS_TEXTURE_VTX	poly[4];

void displayLetter( void *addr, SYE_BT_FNT_FONTSET set, Sint16 letter, Sint16 x, Sint16 y );
void displayChar( void *addr, SYE_BT_FNT_FONTSET set, Sint16 letter, Sint16 x, Sint16 y );
void fillBuffer( Uint32 *addr, Uint32 colour, Sint32 size );
void setPoint( Uint16 *offset, Sint32 x, Sint32 y, Uint16 colour );
void setChar( Sint32 x, Sint32 y, Uint16 colour );


void initFntMode( SMS_FNT_MODE fntMode )
{
	void *bootRomAddr;

	bootRomAddr = syBtFntGet(SYD_BT_FNTINI);

	fntMode->code = 0x88a4;
	fntMode->currentCode = 0x88a4;
	fntMode->cursor = 0;
	fntMode->currentDrawIndex = 0;
	fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_LVL2_24;
	fntMode->mode = SMD_FNT_MODE_CHANGE;
	fntMode->bootRomAddr = bootRomAddr;
	fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_LVL2_24;
}

void initFontTexture( void )
{
    /*ꖇ̃eNX`[ݒ*/
	njSetTextureInfo(&kanjiInfo[0],(Sint8 *) kanjiBuff,NJD_TEXFMT_RGB_565|NJD_TEXFMT_RECTANGLE,KANJI_BUFF_W,KANJI_BUFF_H);
    njSetTextureName(&texname[0], &kanjiInfo[0], 0, NJD_TEXATTR_TYPE_MEMORY | NJD_TEXATTR_GLOBALINDEX);
	njSetTextureInfo(&kanjiInfo[1],(Sint8 *) kanjiBuff,NJD_TEXFMT_RGB_565|NJD_TEXFMT_RECTANGLE,KANJI_BUFF_W,KANJI_BUFF_H);
    njSetTextureName(&texname[1], &kanjiInfo[1], 1, NJD_TEXATTR_TYPE_MEMORY | NJD_TEXATTR_GLOBALINDEX);
    nuLoadTexture(&texlist);
    njSetTexture(&texlist);

	poly[0].x = (640-KANJI_BUFF_W)/2;
	poly[0].y = (480-KANJI_BUFF_H)/2;
	poly[0].z = 0.5;
	poly[0].u = 0;
	poly[0].v = 0;
	poly[0].col = 0xFFFFFFFF;
	
	poly[1].x = poly[0].x;
	poly[1].y = poly[0].y+KANJI_BUFF_H;
	poly[1].z = 0.5;
	poly[1].u = 0;
	poly[1].v = 1.f;
	poly[1].col = 0xFFFFFFFF;
	
	poly[2].x = poly[0].x+KANJI_BUFF_W;
	poly[2].y = poly[0].y;
	poly[2].z = 0.5;
	poly[2].u = 1.f;
	poly[2].v = 0;
	poly[2].col = 0xFFFFFFFF;

	poly[3].x = poly[0].x+KANJI_BUFF_W;
	poly[3].y = poly[0].y+KANJI_BUFF_H;
	poly[3].z = 0.5;
	poly[3].u = 1.f;
	poly[3].v = 1.f;
	poly[3].col = 0xFFFFFFFF;
	kanjiBuffPtr = (Uint16 *)((Uint32)kanjiBuff | 0xa0000000);
}

void chooseFont( SMS_FNT_MODE fntMode )
{
	const PDS_PERIPHERALINFO* info;
	const PDS_PERIPHERAL* per;

	per = pdGetPeripheral(PDD_PORT_A0);

	/* Rg[ڑĂ邩ǂ̔ */
	if (!(per->info->type & PDD_DEVTYPE_CONTROLLER)) {
		return;
	}

	if( per->press & PDD_DGT_KR ) {
		(fntMode->cursor)++;
		if( fntMode->cursor >= 5 ) {
			fntMode->cursor = 0;
		}
	} else {
		if( per->press & PDD_DGT_KL ) {
			if( fntMode->cursor == 0 ) {
				fntMode->cursor = 4;
			} else {
				fntMode->cursor--;
			}
		}
	}

	switch( fntMode->cursor ) {
	case 1:
	case 2:
	case 3:
	case 4:
		if( per->press & PDD_DGT_KU ) {
			if( fntMode->currentCode == 0xffff ) {
				fntMode->currentCode = 0x0000;
			} else {
				fntMode->currentCode+=0x0001 << ((4-fntMode->cursor)*4);
			}
		} else {
			if( per->press & PDD_DGT_KD ) {
				if( fntMode->currentCode == 0x0000 ) {
					fntMode->currentCode = 0xffff;
				} else {
					fntMode->currentCode-=0x0001 << ((4-fntMode->cursor)*4);
				}
			}
		}
		break;
	case 0:
	default:
		if( per->press & PDD_DGT_KU ) {
			switch( fntMode->currentFontset ) {
			case SYE_BT_FNT_FONTSET_WESTERN_24:
				fntMode->currentFontset = SYE_BT_FNT_FONTSET_VMSICON;
				break;
			case SYE_BT_FNT_FONTSET_JP_JIS_24:
				fntMode->currentFontset = SYE_BT_FNT_FONTSET_WESTERN_24;
				break;
			case SYE_BT_FNT_FONTSET_JP_LVL2_24:
				fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_JIS_24;
				break;
			case SYE_BT_FNT_FONTSET_JP_GAIJ_24:
				fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_LVL2_24;
				break;
			case SYE_BT_FNT_FONTSET_VMSICON:
				fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_GAIJ_24;
				break;
			default:
				fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_LVL2_24;
				break;
			}
		} else {
			if( per->press & PDD_DGT_KD ) {
				switch( fntMode->currentFontset ) {
				case SYE_BT_FNT_FONTSET_WESTERN_24:
					fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_JIS_24;
					break;
				case SYE_BT_FNT_FONTSET_JP_JIS_24:
					fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_LVL2_24;
					break;
				case SYE_BT_FNT_FONTSET_JP_LVL2_24:
					fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_GAIJ_24;
					break;
				case SYE_BT_FNT_FONTSET_JP_GAIJ_24:
					fntMode->currentFontset = SYE_BT_FNT_FONTSET_VMSICON;
					break;
				case SYE_BT_FNT_FONTSET_VMSICON:
					fntMode->currentFontset = SYE_BT_FNT_FONTSET_WESTERN_24;
					break;
				default:
					fntMode->currentFontset = SYE_BT_FNT_FONTSET_JP_LVL2_24;
					break;
				}
			}
		}
		break;
	}

	if( per->press & PDD_DGT_TA ) {
		fntMode->code = fntMode->currentCode;
		fntMode->fontset = fntMode->currentFontset;
		fntMode->mode = SMD_FNT_MODE_CHANGE;
	}
}

void displayFont( SMS_FNT_MODE fntMode )
{
	Sint32 chooseX, chooseY;
	Sint32 displayX, displayY;
	char *text[] = {
		"WESTERN_24",			/* 0 */
		"JP_JIS_24",			/* 1 */
		"JP_KANA_24",			/* 2 */
		"JP_LVL1_24",			/* 3 */
		"JP_LVL2_24",			/* 4 */
		"JP_GAIJ_24",			/* 5 */
		"VMSICON_32",			/* 6 */
		"UNKNOWN" };

	chooseX = 3; chooseY = 4;
	displayX = 5; displayY = 19;

	switch( fntMode->currentFontset ) {
	case SYE_BT_FNT_FONTSET_WESTERN_24:
		njPrintC( NJM_LOCATION( chooseX, chooseY ), "*" );
		break;
	case SYE_BT_FNT_FONTSET_JP_JIS_24:
		njPrintC( NJM_LOCATION( chooseX, chooseY+1 ), "*" );
		break;
	case SYE_BT_FNT_FONTSET_JP_LVL2_24:
		njPrintC( NJM_LOCATION( chooseX, chooseY+2 ), "*" );
		break;
	case SYE_BT_FNT_FONTSET_JP_GAIJ_24:
		njPrintC( NJM_LOCATION( chooseX, chooseY+3 ), "*" );
		break;
	case SYE_BT_FNT_FONTSET_VMSICON:
		njPrintC( NJM_LOCATION( chooseX, chooseY+4 ), "*" );
		break;
	default:
		break;
	}

	njPrintC( NJM_LOCATION( chooseX+2, chooseY ), text[0] );
	njPrintC( NJM_LOCATION( chooseX+2, chooseY+1 ), text[1] );
	njPrintC( NJM_LOCATION( chooseX+2, chooseY+2 ), text[4] );
	njPrintC( NJM_LOCATION( chooseX+2, chooseY+3 ), text[5] );
	njPrintC( NJM_LOCATION( chooseX+2, chooseY+4 ), text[6] );

	if( fntMode->cursor == 0 ) {
		njPrintC( NJM_LOCATION( chooseX, chooseY-1 ), "=" );
		njPrintC( NJM_LOCATION( chooseX, chooseY+4+1 ), "=" );
	} else {
		njPrintC( NJM_LOCATION( chooseX+16+1+fntMode->cursor, chooseY+1-1 ), "=" );
		njPrintC( NJM_LOCATION( chooseX+16+1+fntMode->cursor, chooseY+1+1 ), "=" );
	}

	njPrintC( NJM_LOCATION( chooseX+16+1, chooseY+1 ), "[" );
	njPrintH( NJM_LOCATION( chooseX+16+2, chooseY+1 ), fntMode->currentCode, 4 );
	njPrintC( NJM_LOCATION( chooseX+16+6, chooseY+1 ), "]" );

	njPrintC( NJM_LOCATION( displayX, displayY ), " set=" );
	if( fntMode->fontset >= SYE_BT_FNT_FONTSET_END ) {
		njPrintC( NJM_LOCATION( displayX+5, displayY ), text[8] );
	} else {
		njPrintC( NJM_LOCATION( displayX+5, displayY ), text[fntMode->fontset] );
	}
	njPrintC( NJM_LOCATION( displayX, displayY+1 ), "code=" );
	njPrintH( NJM_LOCATION( displayX+5, displayY+1 ), fntMode->code, 4 );

	if( fntMode->mode == SMD_FNT_MODE_CHANGE ) {
		fillBuffer( (Uint32 *)kanjiBuffPtr, COLOUR565_WHITE<<16|COLOUR565_WHITE, KANJI_BUFF_SIZE/4 );
		displayLetter( fntMode->bootRomAddr, fntMode->fontset, fntMode->code, 0, 0 );
		njReLoadTextureLodNum( fntMode->currentDrawIndex?0:1, (Sint8 *) kanjiBuffPtr,0);
		fntMode->currentDrawIndex = fntMode->currentDrawIndex?0:1;
		fntMode->mode = SMD_FNT_MODE_STATIC;
	}

	njSetTextureNum( fntMode->currentDrawIndex );
	njDrawTextureEx( poly, 4, FALSE );

	return;
}

void displayLetter( void *fntAddr, SYE_BT_FNT_FONTSET set, Sint16 letter, Sint16 x, Sint16 y )
{
	Uint8 *addr;
	Sint32 i, j, count;
	SYS_BT_FNT_INFO fontinfo;
	Uint8 c;

	syBtFntGetInfo( set, letter, &fontinfo );
	addr = (Uint8 *)fntAddr;
	addr += syBtFntGetAddr( set, letter );

	if( syBtFntChkSmph() == SYD_BT_FNT_SMPH_SUCCESS ) {
		count = 0;
		for( i=0; count<(fontinfo.height)*(fontinfo.width); i++ ) {
			c = *(addr++);
			for( j=0; j<8; j++ ) {
				setPoint( kanjiBuffPtr, x+(count%fontinfo.width), y+(count/fontinfo.width), ((c & 0x80)!=0)?BACK_COLOUR565:FORE_COLOUR565 );
				c<<=1;
				count++;
				if( count >= (fontinfo.width)*(fontinfo.height) ) {
					break;
				}
			}
		}
		syBtFntClrSmph();
	} else {
		njPrintC( NJM_LOCATION(10, 3), "KANJI SEMAPHORE ERROR!" );
	}

	return;
}

void displayChar( void *fntAddr, SYE_BT_FNT_FONTSET set, Sint16 letter, Sint16 x, Sint16 y )
{
	Uint8 *addr;
	Sint32 i, j, count;
	SYS_BT_FNT_INFO fontinfo;
	Uint8 c;

	syBtFntGetInfo( set, letter, &fontinfo );
	addr = (Uint8 *)fntAddr;
	addr += syBtFntGetAddr( set, letter );

	if( syBtFntChkSmph() == SYD_BT_FNT_SMPH_SUCCESS ) {
		count = 0;
		for( i=0; count<(fontinfo.height)*(fontinfo.width); i++ ) {
			c = *(addr++);
			for( j=0; j<8; j++ ) {
				setChar( x+(count%fontinfo.width), y+(count/fontinfo.width), ((c & 0x80)!=0)?BACK_COLOUR565:FORE_COLOUR565 );
				c<<=1;
				count++;
				if( count >= (fontinfo.width)*(fontinfo.height) ) {
					break;
				}
			}
		}
		syBtFntClrSmph();
	} else {
		njPrintC( NJM_LOCATION(10, 3), "KANJI SEMAPHORE ERROR!" );
	}

	return;
}

void fillBuffer( Uint32 *addr, Uint32 colour, Sint32 size )
{
	Sint32 i;
	for( i=0; i<size; i++ ) {
		*addr++ = colour;
	}
}

void setPoint( Uint16 *offset, Sint32 x, Sint32 y, Uint16 colour )
{
	Uint16 *addr;

	addr = offset + y * KANJI_BUFF_W + x;
	*addr = colour;
}

void setChar( Sint32 x, Sint32 y, Uint16 colour )
{
	if( colour == BACK_COLOUR565 ) {
		njPrintC( NJM_LOCATION(x,y), "0" );
	} else {
		njPrintC( NJM_LOCATION(x,y), "1" );
	}
}
