/********************************************************************
 *  Shinobi Library Sample
 *  Copyright (c) 1998-2000 SEGA
 *
 *  Library : Backup Library
 *  Module  : Library Sample
 *  File    : backup.c
 *  Date    : 2000-02-09
 *  Version : 2.00
 *
 ********************************************************************/

#include <shinobi.h>
#include "backup.h"

/*===============================================================*/
/* Ή郁[J[h̍őe                              */
/* Mamimum volume to use.                                        */
/*===============================================================*/

#define MAX_CAPS BUD_CAPACITY_1MB


/*===============================================================*/
/* Ή郁[J[h̃hCu                            */
/*   [U[th[ȃAvP[Vɂ邽߁A          */
/*   ȂׂׂẴhCugpĂB                */
/* Number of memory card to use.                                 */
/*   You should use all drives in order to make your application */
/*   more friendly(convenient) for users.                        */
/*===============================================================*/

#define MAX_DRIVES 8
#define USE_DRIVES BUD_USE_DRIVE_ALL


/*===============================================================*/
/* [J[h̏Ԃi[Ă\                      */
/* (backup.h Q)                                             */
/* Structure to store the information of memory card.            */
/* (See backup.h)                                                */
/*===============================================================*/

BACKUPINFO gBupInfo[8];


/*===============================================================*/
/* obNAbvCugp郏[N                        */
/* SoCgEŊmۂ                                        */
/* Work buffer that backup library uses.                         */
/* (Must be 4-bytes aligned.)                                    */
/*===============================================================*/

Uint32 gBupWork[BUM_WORK_SIZE(MAX_CAPS, MAX_DRIVES) / sizeof(Uint32)];


/*===============================================================*/
/*      X^eBbN֐̃vg^Cv錾                       */
/*      Prototypes of static functions.                          */
/*===============================================================*/

static Sint32 BupComplete(Sint32 drive, Sint32 op, Sint32 stat, Uint32 param);
static Sint32 BupProgress(Sint32 drive, Sint32 op, Sint32 count, Sint32 max);
static void BupInitCallback(void);


/*===============================================================*/
/*      obNAbvCu                             */
/*      Initialize backup library.                               */
/*===============================================================*/

void BupInit(void)
{
	memset(gBupInfo, 0, sizeof(gBupInfo));
	buInit(MAX_CAPS, USE_DRIVES, gBupWork, BupInitCallback);
}


/*===============================================================*/
/*      obNAbvCuI                               */
/*      Finalize backup library.                                 */
/*===============================================================*/

void BupExit(void)
{
	do {} while (buExit() != BUD_ERR_OK);
}


/*===============================================================*/
/*      [J[h̏Ԃ擾                               */
/*      Get information of memory card.                          */
/*===============================================================*/

const BACKUPINFO* BupGetInfo(Sint32 drive)
{
	return (const BACKUPINFO*)&gBupInfo[drive];
}


/*===============================================================*/
/*      t@C̃[h                                         */
/*      Load a file.                                             */
/*===============================================================*/

Sint32 BupLoad(Sint32 drive, const char* fname, void* buf, BUPFUNC func)
{
	gBupInfo[drive].func = func;
	return buLoadFile(drive, fname, buf, 0);
}


/*===============================================================*/
/*      t@C̃Z[u                                         */
/*      Save a file.                                             */
/*===============================================================*/

Sint32 BupSave(Sint32 drive, const char* fname, void* buf,
										Sint32 nblock, BUPFUNC func)
{
	SYS_RTC_DATE date;

	syRtcGetDate(&date);

	gBupInfo[drive].func = func;
	return buSaveFile(drive, fname, buf, nblock, (BUS_TIME*)&date,
									BUD_FLAG_VERIFY | BUD_FLAG_COPY(0));
}


/*===============================================================*/
/*      t@C̍폜                                           */
/*      Delete a file.                                           */
/*===============================================================*/

Sint32 BupDelete(Sint32 drive, const char* fname, BUPFUNC func)
{
	gBupInfo[drive].func = func;
	return buDeleteFile(drive, fname);
}

/*===============================================================*/
/*      ݒǂ𔻒                                 */
/*      Judge whether writing or not.                            */
/*===============================================================*/

Sint32 BupIsWriting(void)
{
	Sint32 i;
	BACKUPINFO* bup;

	for (i = 0; i < 8; i++) {
		bup = BupGetInfo(i);
		if (bup->Operation == BUD_OP_SAVEFILE) return TRUE;
		if (bup->Operation == BUD_OP_SAVEEXECFILE) return TRUE;
		if (bup->Operation == BUD_OP_REWRITEEXECFILE) return TRUE;
		if (bup->Operation == BUD_OP_DELETEFILE) return TRUE;
		if (bup->Operation == BUD_OP_FORMATDISK) return TRUE;
		if (bup->Operation == BUD_OP_DEFRAGDISK) return TRUE;
		if (bup->Operation == BUD_OP_SETFILEATTR) return TRUE;
		if (bup->Operation == BUD_OP_DELETEFILEEX) return TRUE;
	}

	return FALSE;
}


/*===============================================================*/
/*      G[R[h𕶎ɕϊ                               */
/*      Convert error code into character string.                */
/*===============================================================*/

const char* BupGetErrorString(Sint32 err)
{
	switch (err) {
		case BUD_ERR_OK:				return "OK\0";
		case BUD_ERR_BUSY:				return "BUSY\0";
		case BUD_ERR_INVALID_PARAM:		return "INVALID PARAMETER\0";
		case BUD_ERR_ILLEGAL_DISK:		return "ILLEGAL DISK\0";
		case BUD_ERR_UNKNOWN_DISK:		return "UNKNOWN DISK\0";
		case BUD_ERR_NO_DISK:			return "NO DISK\0";
		case BUD_ERR_UNFORMAT:			return "UNFORMAT\0";
		case BUD_ERR_DISK_FULL:			return "DISK FULL\0";
		case BUD_ERR_FILE_NOT_FOUND:	return "FILE NOT FOUND\0";
		case BUD_ERR_FILE_EXIST:		return "FILE EXIST\0";
		case BUD_ERR_CANNOT_OPEN:		return "CANNOT OPEN\0";
		case BUD_ERR_CANNOT_CREATE:		return "CANNOT CREATE\0";
		case BUD_ERR_EXECFILE_EXIST:	return "EXECUTABLE FILE EXIST\0";
		case BUD_ERR_ACCESS_DENIED:		return "ACCESS DENIED\0";
		case BUD_ERR_VERIFY:			return "VERIFY ERROR\0";
		default:						return "GENERIC ERROR\0";
	}
}


/*===============================================================*/
/*      Iy[VR[h𕶎ɕϊ                       */
/*      Convert operation code into character string.            */
/*===============================================================*/

const char* BupGetOperationString(Sint32 op)
{
	switch (op) {
		case BUD_OP_MOUNT:			return "CONNECTED\0";
		case BUD_OP_UNMOUNT:		return "DISCONNECTED\0";
		case BUD_OP_FORMATDISK:		return "FORMATDISK\0";
		case BUD_OP_DELETEFILE:		return "DELETEFILE\0";
		case BUD_OP_LOADFILE:		return "LOADFILE\0";
		case BUD_OP_SAVEFILE:		return "SAVEFILE\0";
		default:					return "UNKNOWN OPERATION\0";
	}
}


/*===============================================================*/
/*      R[obN֐                                         */
/*      Callback functions.                                      */
/*===============================================================*/

/*
** IR[obN֐
** Callback function when initialization of library was finished.
*/

static void BupInitCallback(void)
{
	Sint32 i;

	/* Ao߃R[obN֐ݒ肷 */
	/* Register callback functions.         */
	buSetCompleteCallback(BupComplete);
	buSetProgressCallback(BupProgress);
}


/*
** R[obN֐
** Callback function when operation was finished.
*/

static Sint32 BupComplete(Sint32 drive, Sint32 op, Sint32 stat, Uint32 param)
{
	BACKUPINFO* info;
	Sint32 ret;

	info = &gBupInfo[drive];

	switch (op) {
		/* [J[hڑꂽ */
		/* Memory card was connected.  */
		case BUD_OP_MOUNT:
			if (stat == BUD_ERR_OK) {
				info->Ready = TRUE;
				ret = buGetDiskInfo(drive, &info->DiskInfo);
				if (ret == BUD_ERR_OK) {
					info->IsFormat = TRUE;
				}
				/* G[ԂNA */
				/* Clear error status */
				info->LastError = BUD_ERR_OK;
			}
			break;
		/* [J[hOꂽ */
		/* Memory card was removed.     */
		case BUD_OP_UNMOUNT:
			info->Ready = FALSE;
			info->IsFormat = FALSE;
			info->ProgressCount = 0;
			info->ProgressMax = 0;
			info->Operation = 0;
			memset(&info->DiskInfo, 0, sizeof(BUS_DISKINFO));
			break;
		default:
			/* G[Ԃi[ */
			/* Store error code. */
			info->LastError = stat;
			/* XV */
			/* Update information of memory card. */
			buGetDiskInfo(drive, &info->DiskInfo);
			/* R[obNĂяo */
			if (info->func) {
				info->func(drive, op, stat);
			}
			break;
	}

	/* Iy[VR[hNA */
	/* Clear operation code.        */
	info->Operation = 0;

	return BUD_CBRET_OK;
}


/*
** o߃R[obN֐
** Callback function through operation progress.
*/

static Sint32 BupProgress(Sint32 drive, Sint32 op, Sint32 count, Sint32 max)
{
	BACKUPINFO* info;

	info = &gBupInfo[drive];

	/* oߏԁAIy[VR[hi[ */
	/* Store operation code.                */
	info->ProgressCount = count;
	info->ProgressMax = max;
	info->Operation = op;

	return BUD_CBRET_OK;
}



/******************************* end of file *******************************/
