/*---------------------------------------------------------------------------
  Shinobi Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.
---------------------------------------------------------------------------*/

#include	<shinobi.h>
#include	<Ninja.h>
#include	"Nj.h"
#include	"gun.h"

/*-------------------------------------------------------------------------*/

void DrawTarget(Sint32 x, Sint32 y, Sint32 gunno);
void DrawRect(Sint32 x, Sint32 y, Sint32 w, Sint32 h, Uint32 color);
void DispGunInfo(Sint32 x, Sint32 y, Sint32 gunno);

void CalibGetPos(Sint32 gunno, Sint32 sx, Sint32 sy, Sint32* adjx, Sint32* adjy);
void CalibChk(Sint32 gunno);
Sint32 CalibStartCheck(void);

Uint32 gFrameCount;
Uint32 gTvMode;
Uint32 gCalibMode;
Uint32 gNowGun;         /* ݂ǂGUNLu[V */


#define CALIB_STOP 0
#define CALIB_MIN  1
#define CALIB_MAX  2
#define CALIB_CEN  3
#define CALIB_SET  4
#define CALIB_CHK  5

static GUNADJ scrX = {-320+32, 0, 320-32};
static GUNADJ scrY = {-240+40, 0, 240-40};
static GUNADJ adjX[4], adjY[4];

static const char* PlayerName[] = {
	"PLAYER 1",
	"PLAYER 2",
	"PLAYER 3",
	"PLAYER 4",
};

static const Uint32 PortNo[] = {
	PDD_PORT_A0,
	PDD_PORT_B0,
	PDD_PORT_C0,
	PDD_PORT_D0,
};

static Uint32 GunBit[] = {
	PDD_GUNMODE_A,
	PDD_GUNMODE_B,
	PDD_GUNMODE_C,
	PDD_GUNMODE_D,
};
/*-------------------------------------------------------------------------*/
Uint32 gFrameCount;

/*---------------------------------------------------------------------------
		
		initialize
---------------------------------------------------------------------------*/
Sint32	UserInit(void)
{
	gFrameCount = 0;
	gCalibMode = CALIB_STOP;
	gNowGun = GUN_A;

	njSetBackColor(0x00004080, 0x00004080, 0x004080);

	/* K̏ */
	GunInit(gTvMode, PDD_GUNMODE_ALL);
	pdGunSetFlashColor(0x00c0c0c0);
	return USER_CONTINUE;
}

/*---------------------------------------------------------------------------
		I
		exit
---------------------------------------------------------------------------*/
Sint32	UserExit(void)
{
	GunExit();

	return USER_CONTINUE;
}

/*---------------------------------------------------------------------------
		C
		main
---------------------------------------------------------------------------*/
Sint32	UserMain(void)
{
	GUNINFO* info;
	GUNADJ* ax;
	GUNADJ* ay;


	njPrintC(NJM_LOCATION( 4, 2), "LIGHT GUN SAMPLE #3");
	njPrintH(NJM_LOCATION(30, 2), gFrameCount, 8);
	njPrint(NJM_LOCATION(  4, 3), "-CALIBRATION %s-", PlayerName[gNowGun]);


	/* Pt[ɂPxĂԃK̃C֐ */
	/* Gun main function                    */
	GunMain();

	ax = &adjX[gNowGun];
	ay = &adjY[gNowGun];

	switch (gCalibMode) {
		case CALIB_STOP:
			/* K̐ڑƃX^[g{^`FbN      */
			/* Check the gun connection and start button */
			CalibStartCheck();
			break;
		case CALIB_MIN:
			/* ʍł̒l擾 */
			/* Get left-top position data */
			CalibGetPos(gNowGun, scrX.min, scrY.min, &ax->min, &ay->min);
			break;
		case CALIB_MAX:
			/* ʉEł̒l擾 */
			/* Get right-bottom position data */
			CalibGetPos(gNowGun, scrX.max, scrY.max, &ax->max, &ay->max);
			break;
		case CALIB_CEN:
			/* ʒł̒l擾 */
			/* Get center position data */
			CalibGetPos(gNowGun, scrX.cen, scrY.cen, &ax->cen, &ay->cen);
			break;
		case CALIB_SET:
			/* ꂽlƂɃLu[Vݒ肷 */
			/* Set the calibration parameters                     */
			GunAdjust(gNowGun, ax, ay, &scrX, &scrY);
			/* \tgEFAgK[hɂ */
			/* Set trigger mode as software trigger mode */
			GunSetRapid(gNowGun, GUN_RAPID_AUTO);
			/* ʃtbVOFFɂ */
			/* Turn off the screen flash */
			pdGunSetFlashColor(0);
			njSetBackColor(0xff808080, 0xff808080, 0xff808080);
			gCalibMode = CALIB_CHK;
			break;
		case CALIB_CHK:
			/* Lu[VʂmF       */
			/* Confirma the result of the calibration */
			CalibChk(gNowGun);
			break;
	}

	/* Lu[ṼKꂽ璆~   */
	/* If the gun is removed, abort the calibration */
	info = GunGetInfo(gNowGun);
	if (!info->IsGun) {
		gCalibMode = CALIB_STOP;
	} else {
		DispGunInfo(2,  20, gNowGun);
	}

	gFrameCount++;

	return USER_CONTINUE;
}




/*===============================================================*/
/* K̏\                                              */
/* Display information of gun controller.                        */
/*===============================================================*/

void DispGunInfo(Sint32 x, Sint32 y, Sint32 gunno)
{
	const GUNINFO* info;

	info = GunGetInfo(gunno);

	if (info->IsGun) {
		/* KڑĂΏ\           */
		/* If gun is connected, displays information. */
		njPrint(NJM_LOCATION(x, y + 4), "(x,y)=(%4d,%4d)",
												info->x, info->y);
		njPrint(NJM_LOCATION(x, y + 5), "raw  =(%4d,%4d)",
											info->rawX, info->rawY);
		njPrint(NJM_LOCATION(x, y + 6), "trigger  = %d", info->trigger);
		njPrint(NJM_LOCATION(x, y + 7), "OnScreen = %d", info->OnScreen);

		/* gKꂽAKʓǂ𔻒肵A */
		/* ʂɕ\                                     */
		/* If trigger was pressed, examines whether there is  */
		/* the muzzle in a screen.                            */
		if (info->trigger) {
			if (info->OnScreen) {
				DrawTarget(info->x + 320, info->y + 240, gunno);
			}
		}
	}
}




/*===============================================================*/
/* ȒPȃ|S`惋[`                                    */
/* Easy polygon rendering routine.                               */
/*===============================================================*/

void DrawTarget(Sint32 x, Sint32 y, Sint32 gunno)
{
	static const Uint32 color[] = {
		0xffffffff, 0xffffff00, 0xffff00ff, 0xff00ffff,
	};

	DrawRect(0, y, 640, 1, color[gunno]);
	DrawRect(x, 0, 1, 480, color[gunno]);
}

void DrawRect(Sint32 x, Sint32 y, Sint32 w, Sint32 h, Uint32 color)
{
	Sint32 i;
	NJS_POLYGON_VTX vtx[4];

	for (i = 0; i < 4; i++) {
		vtx[i].col = color;
		vtx[i].z   = 4.f;
	}

	vtx[0].x = x;
	vtx[0].y = y;
	vtx[2].x = x + w;
	vtx[2].y = y;
	vtx[3].x = x + w;
	vtx[3].y = y + h;
	vtx[1].x = x;
	vtx[1].y = y + h;

	njDrawPolygon(vtx, 4, TRUE);
}

/*===============================================================*/
/* K̐ڑƃX^[g{^̃`FbN                          */
/*   Check the gun connection and start button                   */
/*===============================================================*/

Sint32 CalibStartCheck(void)
{
	Sint32 i;
	const PDS_PERIPHERAL* per;
	GUNINFO* info;

	if (gFrameCount & 0x00000030) {
		njPrintC(NJM_LOCATION( 7, 10), "PRESS START BUTTON TO     ");
		njPrintC(NJM_LOCATION( 7, 11), "         START CALIBRATION");
	}

	for (i = 0; i < 4; i++) {
		per = pdGetPeripheral(PortNo[i]);
		info = GunGetInfo(i);
		if ((per->press & PDD_DGT_ST) && info->IsGun) {
			GunSetPort(GunBit[i]);
			gCalibMode = CALIB_MIN;
			gNowGun = i;
			break;
		}
	}

	return 0;
}


/*===============================================================*/
/* ̍Wł̃Kf[^擾                            */
/*   Get raw data of the gun                                     */
/*===============================================================*/

void CalibGetPos(Sint32 gunno, Sint32 sx, Sint32 sy,
											Sint32* adjx, Sint32* adjy)
{
	Sint32 x, y;
	GUNINFO* info;


	njPrintC(NJM_LOCATION(10, 10), "SHOOT THE TARGET!");
	DrawRect(sx-16+320, sy-16+240, 32, 32, 0xffffffff);

	info = GunGetInfo(gunno);

	if (info->trigger) {
		if ((info->rawX >= 0) && (info->rawY >= 0)) {
			*adjx = info->rawX;
			*adjy = info->rawY;
			gCalibMode++;
		}
	}
}

/*===============================================================*/
/* Lu[VʂmF                              */
/*   Confirm the result of the calibration                       */
/*===============================================================*/

void CalibChk(Sint32 gunno)
{
	const PDS_PERIPHERAL* per;
	GUNADJ* ax;
	GUNADJ* ay;

	if (gFrameCount & 0x00000030) {
		njPrintC(NJM_LOCATION( 7, 10), "PRESS START BUTTON TO EXIT");
		njPrintC(NJM_LOCATION( 7, 11), "     B BUTTON TO RETRY    ");
	}

	ax = &adjX[gunno];
	ay = &adjY[gunno];

	njPrint(NJM_LOCATION(2, 14), "MIN(%d,%d) MAX(%d,%d) CEN(%d,%d)",
		ax->min, ay->min, ax->max, ay->max, ax->cen, ay->cen);

	ax = &scrX;
	ay = &scrY;
	njPrint(NJM_LOCATION(2, 15), "SCREEN(%d,%d)-(%d,%d)",
							ax->min, ay->min, ax->max, ay->max);

	per = pdGetPeripheral(PortNo[gunno]);

	/* START button to exit calibration */
	if (per->press & PDD_DGT_ST) {
		njSetBackColor(0x0000c0c0, 0x0000c0c0, 0x000000c0);
		gCalibMode = CALIB_STOP;
		GunSetRapid(gunno, GUN_RAPID_OFF);
		pdGunSetFlashColor(0x00c0c0c0);
	}

	/* B button retry calibration */
	if (per->press & PDD_DGT_TB) {
		njSetBackColor(0x0000c0c0, 0x0000c0c0, 0x000000c0);
		gCalibMode = CALIB_MIN;
		GunSetRapid(gunno, GUN_RAPID_OFF);
		pdGunSetFlashColor(0x00c0c0c0);
	}
}

/*-------------------------------------------------------------------------*/
