/********************************************************************
 *  Shinobi Library Sample
 *  Copyright (c) 1998-2000 SEGA
 *
 *  Library : Controller Library
 *  Module  : Keyboard Functions
 *  File    : keyboard.h
 *  Date    : 2000-01-17
 *  Version : 2.00
 *  Note    : This file provides a middle level keyboard interface
 *            to your application.
 ********************************************************************/

#ifndef _KEYBOARD_H_
#define _KEYBOARD_H_

#include <shinobi.h>

#define KC_ALT   0x4000
#define KC_SHIFT 0x2000
#define KC_CTRL  0x1000
#define KC_S1    0x0800
#define KC_S2    0x8000

/******** L[{[hǗp\ *******************************************/
/******** Keyboard management structure ************************************/
typedef struct {
	Uint8 used;             /* gptO                                */
	                        /* Used flag                                   */
	Uint8 ready;            /* L[{[hڑĂ邩                */
	                        /* Is keyboard connected?                      */
	Uint8 rate;             /* L[s[gԊu(t[)                */
	                        /* Typematic rate(frame)                       */
	Uint8 delay;            /* s[gJnfBC(t[)            */
	                        /* Typematic delay(frame)                      */
	Uint8 rate_count;       /* s[gp[N                            */
	                        /* Typematic rate work                         */
	Uint8 delay_count;      /* fBCp[N                            */
	                        /* Typematic delay work                        */
	Uint8 repeat_flag;      /* s[gtO                              */
	                        /* Key repeat flag                             */
	Uint8 predata;          /* ÕL[R[h                            */
	                        /* Last key code                               */
	Uint8 old_data[8];      /* ÕL[f[^WoCg                    */
	                        /* Last raw key data (8 bytes)                 */
	Uint32 port;            /* |[gԍ(PDD_PORT_A0/B0/C0/D0)            */
	                        /* Port number(PDD_PORT_A0/B0/C0/D0)           */
	Uint32 putp;            /* L[obt@݃|C^                */
	                        /* Key buffer write pointer                    */
	Uint32 getp;            /* L[obt@ǂݏo|C^                */
	                        /* Key buffer read pointer                     */
	Uint16* buf;            /* L[obt@AhX                        */
	                        /* Key buffer address                          */
	Uint32 bufsize;         /* L[obt@TCY                          */
	                        /* key buffer size                             */
	PDS_KEYBOARD* rawdata;  /* L[{[hf[^ւ̃|C^                */
	                        /* Raw keyboard data pointer                   */
	Uint8** keytbl;         /* L[ϊe[uւ̃|C^                */
	                        /* Key conversion table pointer                */
} KEYBOARD;


/******** L[{[hnh ***********************************************/
/******** Keyboard handle **************************************************/
typedef KEYBOARD* HKBD;


/*
 *  PROTOTYPES
 */

void KbdInit(void);
void kbdSetKeyTable(HKBD hKbs, const Uint8** keytbl);
void KbdSetState(HKBD hKbd, Uint8 rate, Uint8 delay);
void KbdGetState(HKBD hKbd, Uint8* rate, Uint8* delay);
Sint32 KbdIsHit(HKBD hKbd);
Sint32 KbdIsReady(HKBD hKbd);
HKBD KbdCreate(Uint32 port);
void KbdDestroy(HKBD kbd);
Uint16 KbdGetChar(HKBD hKbd);
const PDS_KEYBOARD* KbdGetRawData(HKBD kbd);
void KbdServer(void);


#endif /* _KEYBOARD_H_ */
