/*---------------------------------------------------------------------------
  Shinobi Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.
---------------------------------------------------------------------------*/

#include	<shinobi.h>
#include	<Ninja.h>
#include	"Nj.h"
#include	"dbgcons.h"
#include	<sg_pdms.h>

/*-------------------------------------------------------------------------*/
#define TEXT_W   36           /* R\[̉ Console width  */
#define TEXT_H   16           /* R\[̍s Console height */


/*-------------------------------------------------------------------------*/
void PrintMouseInfo(Sint32 x, Sint32 y, Sint32 port);
void DrawRect(Sint32 x, Sint32 y, Sint32 w, Sint32 h, Uint32 color);
Sint32 search_mouse(void);

/*-------------------------------------------------------------------------*/
Uint32 gFrameCount;

Sint32 gMousePort;
Sint32 gMouseX, gMouseY;
char TextBuf[TEXT_W * TEXT_H];


/*-------------------------------------------------------------------------*/
/* VBLANK荞݊֐ */
void user_vsync(void)
{
	PDS_MOUSE_AXIS* pa;

	pa = pdMouseGetAxis(gMousePort);
	if (!(pa->over & PDD_MOUSE_AXIS_X)) gMouseX += pa->x;
	if (!(pa->over & PDD_MOUSE_AXIS_Y)) gMouseY += pa->y;

	if (gMouseX < 0)   gMouseX = 0;
	if (gMouseY < 0)   gMouseY = 0;
	if (gMouseX > 639) gMouseX = 639;
	if (gMouseY > 479) gMouseY = 479;
}

/*---------------------------------------------------------------------------
		
		initialize
---------------------------------------------------------------------------*/
Sint32	UserInit(void)
{
	gFrameCount = 0;
	gMousePort = -1;
	gMouseX = 320;
	gMouseY = 240;
	njSetBackColor(0x00004080, 0x00004080, 0x004080);

	ConInit(2, 10, TEXT_W, TEXT_H, TextBuf);

	return USER_CONTINUE;
}

/*---------------------------------------------------------------------------
		I
		exit
---------------------------------------------------------------------------*/
Sint32	UserExit(void)
{
	return USER_CONTINUE;
}

/*---------------------------------------------------------------------------
		C
		main
---------------------------------------------------------------------------*/
Sint32	UserMain(void)
{
	PDS_MOUSE_BUTTON* pb;

	njPrintC(NJM_LOCATION(4,  2), "MOUSE SAMPLE #1");
	njPrintH(NJM_LOCATION(20,  2), gFrameCount, 8);

	gMousePort = search_mouse();

	if (gMousePort < 0) {
		njPrint(NJM_LOCATION(10, 5), "PLEASE CONNECT MOUSE");
	} else {
		pb = pdMouseGetButton(gMousePort);
		PrintMouseInfo( 2, 5, gMousePort);	/* Port B */
		njPrint(NJM_LOCATION(4, 8), "(%3d,%3d)", gMouseX, gMouseY);
		DrawRect(gMouseX, gMouseY, 16, 16, 0xffffffff);
	}

	ConDispCursor();
	ConMain();

	user_vsync();

	gFrameCount++;

	return USER_CONTINUE;
}



/*-------------------------------------------------------------------------*/
/* |S`惋[` */
void DrawRect(Sint32 x, Sint32 y, Sint32 w, Sint32 h, Uint32 color)
{
	Sint32 i;
	NJS_POLYGON_VTX vtx[4];

	for (i = 0; i < 4; i++) {
		vtx[i].col = color;
		vtx[i].z   = 4.f;
	}

	vtx[0].x = x;
	vtx[0].y = y;
	vtx[2].x = x + w;
	vtx[2].y = y;
	vtx[3].x = x + w;
	vtx[3].y = y + h;
	vtx[1].x = x;
	vtx[1].y = y + h;

	njDrawPolygon(vtx, 4, TRUE);
}

/*---------------------------------------------------------------------------
		ytF̕\
		print peripheral info
---------------------------------------------------------------------------*/
void PrintMouseInfo(Sint32 x, Sint32 y, Sint32 port)
{
	PDS_MOUSE_BUTTON* pb;
	PDS_MOUSE_AXIS* pa;
	PDS_MOUSE_INFO* pi;
	static const char* over[] = {"X", "Y", "Z", "A3", "A4", "A5", "A6", "A7"};

	pb = pdMouseGetButton(port);
	pa = pdMouseGetAxis(port);
	pi = pdMouseGetInfo(port);

	njPrint(NJM_LOCATION(x, y++),
				"ON=%02X/OFF=%02X/PRS=%02X/RLS=%02X/OPT=%02X",
					pb->on, pb->off, pb->press, pb->release, pb->opt);

	njPrint(NJM_LOCATION(x, y), "OVR=%02X/",pa->over);

	njPrint(NJM_LOCATION(x + 7, y++),
		"AXES=%3d %3d %3d %3d %3d %3d %3d %3d",
			pa->x, pa->y, pa->z,
			pa->axis[0], pa->axis[1],
			pa->axis[2], pa->axis[3],
			pa->axis[4], pa->axis[5]);

	njPrint(NJM_LOCATION(x, y++), "BTN=%02X/AXS=%02X",
								pi->button, pi->axis);

	if (pa->over) {
		Sint32 i;
		Uint8 bit = 1;
		for (i = 0; i < 8; i++, bit <<= 1)
		if (bit & pa->over) ConPrintf("%S OVER!!!\n", over[i]);
	}

	if (pb->press & PDD_MOUSE_BUTTON_L) {
		ConPrintf("Left Button pressed.\n");
	}
	if (pb->press & PDD_MOUSE_BUTTON_R) {
		ConPrintf("Right Button pressed.\n");
	}
	if (pb->press & PDD_MOUSE_BUTTON_M) {
		ConPrintf("Middle Button pressed.\n");
	}
	if (pb->press & PDD_MOUSE_BUTTON_S) {
		ConPrintf("Wheel Button pressed.\n");
	}

	if (pa->z) {
		ConPrintf("Wheel=%d\n", pa->z);
	}
}


/*-------------------------------------------------------------------------*/
Sint32 search_mouse(void)
{
	Sint32 i;
	static const Uint32 port[] = {
		PDD_PORT_A0, PDD_PORT_B0, PDD_PORT_C0, PDD_PORT_D0,
	};

	for (i = 0; i < 4; i++) {
		if (pdMouseIsReady(port[i])) return port[i];
	}

	return -1;
}
