/*---------------------------------------------------------------------------
  Shinobi Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.
---------------------------------------------------------------------------*/

#include	<shinobi.h>
#include	<Ninja.h>
#include	"Nj.h"

/*-------------------------------------------------------------------------*/
void PrintPeripheralCaps(Sint32 x, Sint32 y, Uint32 port);
void PrintPeripheralInfo(Sint32 x, Sint32 y, Uint32 port);

/*-------------------------------------------------------------------------*/
Uint32 gFrameCount;
Sint32 index;


/*===============================================================*/
/* |[gԍi[z                                      */
/* Array of control port number.                                 */
/*===============================================================*/

const Sint32 PortNo[] = {
	PDD_PORT_A0, PDD_PORT_A1, PDD_PORT_A2,
	PDD_PORT_B0, PDD_PORT_B1, PDD_PORT_B2,
	PDD_PORT_C0, PDD_PORT_C1, PDD_PORT_C2,
	PDD_PORT_D0, PDD_PORT_D1, PDD_PORT_D2,
};


/*===============================================================*/
/* e핶̒`                                              */
/* Character strings.                                            */
/*===============================================================*/

const char* PortName[] = {
	"A-0", "A-1", "A-2",
	"B-0", "B-1", "B-2",
	"C-0", "C-1", "C-2",
	"D-0", "D-1", "D-2",
};

const char* TypeName[] = {
	"CONTROLLER", "STORAGE", "LCD", "TIMER",
	"SOUNDINPUT", "?", "KEYBOARD", "LIGHT-GUN",
	"VIBRATION", "POINTING", "?", "CAMERA",
	"FLASHCARD", "?", "?", "?",
};

const char* AreaName[] = {
	"USA", "JAPAN", "ASIA", "EUROPE",
	"RSVD1", "RSVD2", "RSVD3", "RSVD4",
};

const char* DirName[] = {
	"TOP", "BOTTOM", "LEFT", "RIGHT",
};



/*---------------------------------------------------------------------------
		
		initialize
---------------------------------------------------------------------------*/
Sint32	UserInit(void)
{
	gFrameCount = 0;
	njSetBackColor(0x00004080, 0x00004080, 0x004080);

	return USER_CONTINUE;
}

/*---------------------------------------------------------------------------
		I
		exit
---------------------------------------------------------------------------*/
Sint32	UserExit(void)
{
	return USER_CONTINUE;
}

/*---------------------------------------------------------------------------
		C
		main
---------------------------------------------------------------------------*/
Sint32	UserMain(void)
{
	const PDS_PERIPHERALINFO* info;
	const PDS_PERIPHERAL* per;

	njPrintC(NJM_LOCATION(4,  2), "PERIPHERAL INFO SAMPLE");
	njPrintH(NJM_LOCATION(30, 2), gFrameCount, 8);

	per = pdGetPeripheral(PDD_PORT_A0);

	/* Rg[ڑĂ邩ǂ̔ */
	/* Is controller connected to port-A ?        */
	if (!(per->info->type & PDD_DEVTYPE_CONTROLLER)) {
		if (gFrameCount & 0x00000020) {
			njPrintC(NJM_LOCATION(1, 26),
				"PLEASE CONNECT A CONTROLLER TO PORT A.");
		}
	} else {
		njPrintC(NJM_LOCATION(1, 26),
				"PRESS <- -> KEY TO SELECT PERIPHERAL.");
	}

	if (per->press & PDD_DGT_KR) index++;
	if (per->press & PDD_DGT_KL) index--;
	if (index < 0) index = 11;
	if (index > 11) index = 0;

	PrintPeripheralCaps(1, 4, PortNo[index]);

	switch (PortNo[index]) {
		case PDD_PORT_A0:
		case PDD_PORT_B0:
		case PDD_PORT_C0:
		case PDD_PORT_D0:
			PrintPeripheralInfo(1, 13, PortNo[index]);
			break;
		default:
			break;
	}

	gFrameCount++;

	return USER_CONTINUE;
}



/*===============================================================*/
/* ytF̌ŗL\                                  */
/* Display information of peripheral.                            */
/*===============================================================*/

void PrintPeripheralCaps(Sint32 x, Sint32 y, Uint32 port)
{
	Sint32 i, bit, xx;
	const PDS_PERIPHERALINFO* info;

	info = pdGetPeripheralInfo(port);

	njPrint(NJM_LOCATION(x, y++), "PORT NO.:%s",      PortName[index]);
	njPrint(NJM_LOCATION(x, y++), "TYPE    :[%08X]",  info->type);

	xx = x + 20;
	for (i = 0, bit = 1; i < 16; i++, bit <<= 1) {
		if (info->type & bit) {
			njPrint(NJM_LOCATION(xx, y), "%s", TypeName[i]);
			xx += strlen(TypeName[i]) + 1;
		}
	}
	y++;
	njPrint(NJM_LOCATION(x, y++), "NAME    :%s",      info->product_name);
	njPrint(NJM_LOCATION(x, y++), "LICENSE :%s",      info->license);
	njPrint(NJM_LOCATION(x, y),   "AREA    :[%02X]",  info->area_code);

	xx = x + 15;
	for (i = 0, bit = 1; i < 4; i++, bit <<= 1) {
		if (info->area_code & bit) {
			njPrint(NJM_LOCATION(xx, y), "%s", AreaName[i]);
			xx += strlen(AreaName[i]) + 1;
		}
	}

	y++;
	njPrint(NJM_LOCATION(x, y), "CONNECT :[%d %d]", info->connector_dir[0],
                                                    info->connector_dir[1]);
	njPrint(NJM_LOCATION(x + 15, y++), "%s %s",
					DirName[info->connector_dir[0]],
					DirName[info->connector_dir[1]]);

	njPrint(NJM_LOCATION(x, y++), "STANDBY :%dmA", info->stdby_pow / 10);
	njPrint(NJM_LOCATION(x, y++), "MAX POW :%dmA", info->max_pow / 10);
}



/*===============================================================*/
/* ytF̏\                                      */
/* Display information of peripheral.                            */
/*===============================================================*/

void PrintPeripheralInfo(Sint32 x, Sint32 y, Uint32 port)
{
	const PDS_PERIPHERAL* per;

	per = pdGetPeripheral(port);

	njPrintC(NJM_LOCATION(x, y +  0), "SUPPORT:");
	njPrintC(NJM_LOCATION(x, y +  1), "ON     :");
	njPrintC(NJM_LOCATION(x, y +  2), "OFF    :");
	njPrintC(NJM_LOCATION(x, y +  3), "PRESS  :");
	njPrintC(NJM_LOCATION(x, y +  4), "RELEASE:");
	njPrintC(NJM_LOCATION(x, y +  5), "R      :");
	njPrintC(NJM_LOCATION(x, y +  6), "L      :");
	njPrintC(NJM_LOCATION(x, y +  7), "X1     :");
	njPrintC(NJM_LOCATION(x, y +  8), "Y1     :");
	njPrintC(NJM_LOCATION(x, y +  9), "X2     :");
	njPrintC(NJM_LOCATION(x, y + 10), "Y2     :");

	x += 8;
	njPrintH(NJM_LOCATION(x, y), per->support, 8);
	njPrintH(NJM_LOCATION(x, y + 1), per->on, 8);
	njPrintH(NJM_LOCATION(x, y + 2), per->off, 8);
	njPrintH(NJM_LOCATION(x, y + 3), per->press, 8);
	njPrintH(NJM_LOCATION(x, y + 4), per->release, 8);
	njPrintD(NJM_LOCATION(x + (per->r  >= 0), y + 5), per->r, 5);
	njPrintD(NJM_LOCATION(x + (per->l  >= 0), y + 6), per->l, 5);
	njPrintD(NJM_LOCATION(x + (per->x1 >= 0), y + 7), per->x1, 5);
	njPrintD(NJM_LOCATION(x + (per->y1 >= 0), y + 8), per->y1, 5);
	njPrintD(NJM_LOCATION(x + (per->x2 >= 0), y + 9), per->x2, 5);
	njPrintD(NJM_LOCATION(x + (per->y2 >= 0), y + 10), per->y2, 5);
}

